@@ -1209,8 +1209,8 @@
 ("the primitive TeXmacs constructs" "las construcciones primitivas de TeXmacs")
 ("the standard TeXmacs styles" "los estilos estndar de TeXmacs")
 ("theorem" "teorema")
-("there are unsaved files" "hay ficheros sin guardar")
-("this buffer has not been saved" "no se ha grabado este bfer")
+("there are unsaved documents" "hay ficheros sin guardar")
+("this document has not been saved" "no se ha grabado este bfer")
 ("this document has been produced using" "este documento ha sido producido utilizando")
 ("this line" "esta lnea")
 ("this page" "esta pgina")
