@@ -1,4 +1,4 @@
-for
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
 ;; MODULE      : text-menu.scm
@@ -14,7 +14,8 @@ for
 (texmacs-module (text text-menu)
   (:use (text text-edit)
         (text text-structure)
-        (generic document-menu)))
+        (generic document-menu)
+	(generic document-style)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; The Format menu in text mode
@@ -37,8 +38,9 @@ for
   (group "Paragraph")
   (link paragraph-menu)
   ---
-  (group "Page")
-  (link page-menu))
+  (when (in-main-flow?)
+    (group "Page")
+    (link page-menu)))
 
 (menu-bind compressed-text-format-menu
   (if (new-fonts?)
@@ -46,20 +48,30 @@ for
   (if (not (new-fonts?))
       (-> "Font" (link text-font-menu)))
   ("Paragraph" (open-paragraph-format))
-  ("Page" (open-page-format))
+  (when (in-main-flow?)
+    ("Page" (open-page-format)))
+  (when (inside? 'table)
+    ("Cell" (open-cell-properties))
+    ("Table" (open-table-properties)))
   ---
   (-> "Whitespace" (link space-menu))
   (-> "Indentation" (link indentation-menu))
   (-> "Break" (link break-menu))
+  (when (and (selection-active-small?) (tm-atomic? (selection-tree)))
+    ("Hyphenate as" (interactive hyphenate-selection-as)))
   ---
   (-> "Color"
       (if (== (get-preference "experimental alpha") "on")
-	  (-> "Opacity" (link opacity-menu))
-	  ---)
+          (-> "Opacity" (link opacity-menu))
+          ---)
       (link color-menu))
   (-> "Adjust" (link adjust-menu))
+  (-> "Transform" (link linear-transform-menu))
   (-> "Specific" (link specific-menu))
-  (-> "Special" (link format-special-menu)))
+  (-> "Special" (link format-special-menu))
+  (-> "Font effects" (link text-font-effects-menu))
+  (assuming (== (get-preference "bitmap effects") "on")
+    (-> "Graphical effects" (link text-effects-menu))))
 
 (menu-bind text-format-menu
   (if (use-menus?)
@@ -73,23 +85,23 @@ for
 
 (menu-bind title-menu
   (when (not (inside? 'doc-data))
-	("Insert title" (make-doc-data)))
+        ("Insert title" (make-doc-data)))
   (when (and (not (inside? 'doc-data)) (not (inside? 'abstract-data)))
-	("Abstract" (make-abstract-data))))
+        ("Abstract" (make-abstract-data))))
 
 (menu-bind letter-header-menu
   (when (not (inside? 'letter-header))
-	("Header" (make 'letter-header)))
+        ("Header" (make 'letter-header)))
   (when (inside? 'letter-header)
-	("Address" (make-header 'address))
-	("Date" (make-header 'letter-date))
-	("Today" (begin (make-header 'letter-date) (make-arity 'date 0)))
-	("Destination" (make-header 'destination)))
+        ("Address" (make-header 'address))
+        ("Date" (make-header 'letter-date))
+        ("Today" (begin (make-header 'letter-date) (make 'date 0)))
+        ("Destination" (make-header 'destination)))
   ---
   (when (not (inside? 'letter-header))
-	("Opening" (make 'opening))
-	("Closing" (make 'closing))
-	("Signature" (make 'signature)))
+        ("Opening" (make 'opening))
+        ("Closing" (make 'closing))
+        ("Signature" (make 'signature)))
   ---
   ("C.C." (make 'cc))
   ("Encl." (make 'encl)))
@@ -115,7 +127,7 @@ for
   ("Subsection" (make-section 'subsection))
   ("Subsubsection" (make-section 'subsubsection))
   ---
-  ("Paragraph" (make-section 'paragraph))
+  ("Paragraph::section" (make-section 'paragraph))
   ("Subparagraph" (make-section 'subparagraph)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -143,9 +155,9 @@ for
       ---)
   ("Exercise" (make 'exercise))
   ("Problem" (make 'problem))
-  ("Solution" (make 'solution))
   ("Question" (make 'question))
-  ("Answer" (make 'answer)))
+  ("Solution" (make 'solution*))
+  ("Answer" (make 'answer*)))
 
 (menu-bind prominent-menu
   ("Quote" (make 'quote-env))
@@ -154,11 +166,17 @@ for
   ---
   ("Indent" (make 'indent))
   ("Jump in" (make 'jump-in))
-  ("Compact" (make 'compact))
   ---
-  ("Centered" (make 'center))
-  ("Left aligned" (make 'left-aligned))
-  ("Right aligned" (make 'right-aligned))
+  ("Centered" (make 'padded-center))
+  ("Left aligned" (make 'padded-left-aligned))
+  ("Right aligned" (make 'padded-right-aligned))
+  (with s (get-env "par-par-sep")
+    (assuming (and (not (string-ends? s "fns"))
+                   (not (string-starts? s "0fn")))
+      ---
+      ("Compact" (make 'compact))
+      ("Compressed" (make 'compressed))
+      ("Amplified" (make 'amplified))))
   ---
   ("Padded" (make 'padded))
   ("Overlined" (make 'overlined))
@@ -166,69 +184,182 @@ for
   ("Lines around" (make 'bothlined))
   ("Framed" (make 'framed))
   ("Ornamented" (make 'ornamented))
-  (if (style-has? "ornaments-dtd")
+  ---
+  (-> "Material"
+      ("Manila paper" (make* 'manila-paper "ornaments"))
+      ("Rough paper" (make* 'rough-paper "ornaments"))
+      ("Ridged paper" (make* 'ridged-paper "ornaments"))
+      ("Pine" (make* 'pine "ornaments"))
+      ("Granite" (make* 'granite "ornaments"))
+      ("Metal" (make* 'metal "ornaments")))
+  (-> "Art frame"
+      ("Carved wood" (make* 'carved-wood-frame "std-frame"))
+      ("Decorated wood" (make* 'decorated-wood-frame "std-frame"))
+      ("Black floral I" (make* 'black-floral1-frame "std-frame"))
+      ("Black floral II" (make* 'black-floral2-frame "std-frame")))
+  (-> "Shadow"
+      (group "Contour")
+      ("Drop" (make* 'drop-contour "std-shadow"))
+      ("Bend in" (make* 'bend-in-contour "std-shadow"))
+      ("Bend out" (make* 'bend-out-contour "std-shadow"))
+      ("Bend diagonal" (make* 'diagonal-bend-in-contour "std-shadow"))
       ---
-      ("Manila paper" (make 'manila-paper))
-      ("Rough paper" (make 'rough-paper))
-      ("Ridged paper" (make 'ridged-paper))
-      ("Pine" (make 'pine))
-      ("Granite" (make 'granite))
-      ("Metal" (make 'metal))))
+      (group "South east")
+      ("Drop" (make* 'drop-shadow "std-shadow"))
+      ("Bend in" (make* 'bend-in-shadow "std-shadow"))
+      ("Bend out" (make* 'bend-out-shadow "std-shadow"))
+      ("Bend bottom" (make* 'half-bend-in-shadow "std-shadow"))
+      ("Bend corner" (make* 'quarter-bend-in-shadow "std-shadow"))
+      ---
+      (group "South")
+      ("Drop" (make* 'drop-down "std-shadow"))
+      ("Bend bottom" (make* 'half-bend-in-down "std-shadow"))))
 
 (menu-bind code-menu
   ("Algorithm" (make 'algorithm))
   ("Pseudo code" (make 'render-code))
   ---
   ("Indent" (make 'indent))
-  ("Tabbed" (make 'tabbed))
+  (when (not (selection-active?))
+    ("Tabbed" (make 'wide-tabbed)))
   ---
   (-> "Inline code"
       ("C++" (make 'cpp))
+      ("Fortran" (make 'fortran))
+      ("Java" (make 'java))
       ("Mathemagix" (make 'mmx))
+      ("Python" (make 'python))
+      ("R" (make 'r))
+      ("Scala" (make 'scala))
       ("Scheme" (make 'scm))
       ("Shell" (make 'shell))
       ("Scilab" (make 'scilab))
       ("Verbatim" (make 'verbatim)))
   (-> "Block of code"
       ("C++" (make 'cpp-code))
+      ("Fortran" (make 'fortran-code))
+      ("Java" (make 'java-code))
       ("Mathemagix" (make 'mmx-code))
+      ("Python" (make 'python-code))
+      ("R" (make 'r-code))
+      ("Scala" (make 'scala-code))
       ("Scheme" (make 'scm-code))
       ("Shell" (make 'shell-code))
       ("Scilab" (make 'scilab-code))
       ("Verbatim" (make 'verbatim-code))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; Floating objects
+;; Notes and floating objects
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (menu-bind note-menu
   (when (not (or (inside? 'float) (inside? 'footnote)))
-    ("Footnote" (make 'footnote))
+    (when (in-main-flow?)
+      ("Footnote" (make-wrapped 'footnote)))
+    (when (not (selection-active-non-small?))
+      ("Marginal note" (make-marginal-note)))
+    ("Balloon" (make-balloon))
     ---
-    ("Floating object" (make-insertion "float"))
-    ("Floating figure" (begin (make-insertion "float") (make 'big-figure)))
-    ("Floating table" (begin (make-insertion "float") (make 'big-table)))
-    ("Floating algorithm" (begin (make-insertion "float") (make 'algorithm)))))
+    (when (in-main-flow?)
+      ("Floating object" (make-insertion "float"))
+      (when (not (selection-active-non-small?))
+        ("Floating phantom" (insert '(phantom-float "float" "hf"))))
+      (when (not (selection-active-non-small?))
+        ("Floating figure"
+         (wrap-selection-small
+           (make-insertion "float")
+           (insert-go-to '(big-figure "" (document "")) '(0 0))))
+        ("Floating table"
+         (wrap-selection-small
+           (make-insertion "float")
+           (insert-go-to '(big-table "" (document "")) '(0 0))
+           (make 'tabular))))
+      ("Floating algorithm"
+       (wrap-selection-any
+         (make-insertion "float")
+         (make 'algorithm))))))
 
-(menu-bind position-float-menu
+(menu-bind position-marginal-note-menu
+  (group "Horizontal position")
+  ("Automatic" (set-marginal-note-hpos "normal"))
+  ("Left" (set-marginal-note-hpos "left"))
+  ("Right" (set-marginal-note-hpos "right"))
+  ("Left on even pages" (set-marginal-note-hpos "even-left"))
+  ("Right on even pages" (set-marginal-note-hpos "even-right"))
+  ---
+  (group "Vertical alignment")
+  ("Top" (set-marginal-note-valign "t"))
+  ("Center" (set-marginal-note-valign "c"))
+  ("Bottom" (set-marginal-note-valign "b")))
+
+(tm-define (marginal-note-context? t)
+  (tree-is? t 'marginal-note))
+
+(tm-menu (focus-float-menu t)
+  (:require (marginal-note-context? t))
+  ---
+  (link position-marginal-note-menu)
+  ---)
+
+(menu-bind float-menu
   ("Top" (toggle-insertion-positioning "t"))
   ("Here" (toggle-insertion-positioning "h"))
   ("Bottom" (toggle-insertion-positioning "b"))
-  ("Other pages" (toggle-insertion-positioning-not "f")))
+  ("Other pages" (toggle-insertion-positioning-not "f"))
+  (if (tree-innermost float-context? #t)
+      ---
+      ("Make non floating"
+       (turn-non-floating (tree-innermost float-context? #t)))))
 
-(tm-define (float-context? t)
-  (cond ((tree-is? t 'float) #t)
-        ((tree-in? t '(big-figure big-figure*
-                       big-table big-table*
-                       algorithm algorithm*
-                       document concat))
-         (and (tree-up t) (float-context? (tree-up t))))
-        (else #f)))
+(tm-menu (focus-float-menu t)
+  (:require (rich-float-context? t))
+  (if (in-multicol-style?)
+      ("Wide float" (float-toggle-wide (focus-tree))))
+  (-> "Allowed positions" (link float-menu))
+  (if (cursor-at-anchor?)
+      ("Go to float" (go-to-float)))
+  (if (not (cursor-at-anchor?))
+      ("Go to anchor" (go-to-anchor))))
 
-(tm-menu (focus-position-float-menu t)
-  (:require (float-context? t))
-  (-> "Allowed positions"
-      (link position-float-menu)))
+(tm-menu (focus-float-menu t)
+  (:require (phantom-float-context? t))
+  (-> "Allowed positions" (link float-menu)))
+
+(tm-menu (focus-float-menu t)
+  (:require (floatable-context? t))
+  (if (in-multicol-style?)
+      ("Span over all columns" (floatable-toggle-wide t)))
+  ("Make floating" (turn-floating (tree-innermost floatable-context?))))
+
+(tm-menu (focus-float-menu t)
+  (:require (tree-is? t 'footnote))
+  (if (in-multicol-style?)
+      ("Wide footnote" (float-toggle-wide (focus-tree))))
+  (if (cursor-at-anchor?)
+      ("Go to footnote" (go-to-float)))
+  (if (not (cursor-at-anchor?))
+      ("Go to anchor" (go-to-anchor))))
+
+(menu-bind position-balloon-menu
+  (group "Horizontal alignment")
+  ("Outer left" (set-balloon-halign "Left"))
+  ("Inner left" (set-balloon-halign "left"))
+  ("Center" (set-balloon-halign "center"))
+  ("Inner right" (set-balloon-halign "right"))
+  ("Outer right" (set-balloon-halign "Right"))
+  ---
+  (group "Vertical alignment")
+  ("Outer bottom" (set-balloon-valign "Bottom"))
+  ("Inner bottom" (set-balloon-valign "bottom"))
+  ("Center" (set-balloon-valign "center"))
+  ("Inner top" (set-balloon-valign "top"))
+  ("Outer top" (set-balloon-valign "Top")))
+
+(tm-menu (focus-float-menu t)
+  (:require (balloon-context? t))
+  ---
+  (link position-balloon-menu)
+  ---)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Tags
@@ -243,7 +374,8 @@ for
   ("Name" (make 'name))
   ("Person" (make 'person))
   ("Cite" (make 'cite*))
-  ("Abbreviation" (make 'abbr))
+  (when (not (selection-active-non-small?))
+    ("Abbreviation" (make 'abbr)))
   ("Acronym" (make 'acronym))
   ---
   ("Verbatim" (make 'verbatim))
@@ -320,7 +452,8 @@ for
   ("Compact" (make-tmlist 'description-compact))
   ("Aligned" (make-tmlist 'description-aligned))
   ("Dashes" (make-tmlist 'description-dash))
-  ("Long" (make-tmlist 'description-long)))
+  ("Long" (make-tmlist 'description-long))
+  ("Paragraphs" (make-tmlist 'description-paragraphs)))
 
 (menu-bind list-menu
   ("Itemize" (make-tmlist 'itemize))
@@ -342,30 +475,43 @@ for
   ("Compact" (make-tmlist 'description-compact))
   ("Aligned" (make-tmlist 'description-aligned))
   ("Dashes" (make-tmlist 'description-dash))
-  ("Long" (make-tmlist 'description-long)))
+  ("Long" (make-tmlist 'description-long))
+  ("Paragraphs" (make-tmlist 'description-paragraphs)))
+
+(menu-bind lists-menu
+  (-> "Itemize" (link itemize-menu))
+  (-> "Enumerate" (link enumerate-menu))
+  (-> "Description" (link description-menu)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Automatically generated content
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (menu-bind automatic-menu
-  ("Table of contents" (make-aux "table-of-contents" "toc"))
-  ("Bibliography" (interactive make-bib))
-  ("Index" (make-aux "the-index" "idx"))
-  ("Glossary" (make-aux "the-glossary" "gly"))
-  ;;("List of figures" (make-aux* "the-glossary*" "figure" "List of figures"))
-  ;;("List of tables" (make-aux* "the-glossary*" "table" "List of tables"))
-  ("List of figures" (make-aux "list-of-figures" "figure"))
-  ("List of tables" (make-aux "list-of-tables" "table")))
+  ("Table of contents" (make-aux "table-of-contents" "toc-prefix" "toc"))
+  (assuming (get-boolean-preference "gui:new bibliography dialogue")
+    ("Bibliography" (interactive open-bibliography-inserter)))
+  (assuming (not (get-boolean-preference "gui:new bibliography dialogue"))
+    (if (with-database-tool?)
+        ("Bibliography" (make-database-bib)))
+    (if (not (with-database-tool?))
+        ("Bibliography" (choose-file make-bib "Bibliography file" "BibTeX"))))
+  ("Index" (make-aux "the-index" "index-prefix" "idx"))
+  ("Glossary" (make-aux "the-glossary" "glossary-prefix" "gly"))
+  ;;("List of figures" (make-aux* "the-glossary*" "figure-list-prefix" "figure" "List of figures"))
+  ;;("List of tables" (make-aux* "the-glossary*" "table-list-prefix" "table" "List of tables"))
+  ("List of figures" (make-aux "list-of-figures" "figure-list-prefix" "figure"))
+  ("List of tables" (make-aux "list-of-tables" "table-list-prefix" "table")))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; Style dependent menus
+;; Text menus for inserting block content
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(menu-bind text-menu
+(menu-bind text-block-menu
   (if (and (style-has? "header-title-dtd")
-	   (not (style-has? "header-letter-dtd"))
-	   (not (style-has? "header-exam-dtd")))
+           (not (style-has? "header-letter-dtd"))
+           (not (style-has? "header-exam-dtd"))
+           (not (in-poster?)))
       (-> "Title" (link title-menu)))
   (if (style-has? "header-letter-dtd")
       (-> "Header" (link letter-header-menu)))
@@ -374,7 +520,7 @@ for
   (if (style-has? "book-style")
       (-> "Chapter" (link chapter-menu)))
   (if (and (style-has? "section-base-dtd")
-	   (not (style-has? "header-exam-dtd")))
+           (not (style-has? "header-exam-dtd")))
       (-> "Section" (link section-menu)))
   (if (or (style-has? "env-theorem-dtd")
           (style-has? "header-exam-dtd"))
@@ -389,28 +535,87 @@ for
       (-> "Automatic" (link automatic-menu)))
   (if (style-has? "std-list-dtd")
       ---
-      (-> "Itemize" (link itemize-menu))
-      (-> "Enumerate" (link enumerate-menu))
-      (-> "Description" (link description-menu)))
-  ---
+      (link lists-menu)))
+
+(menu-bind text-extra-menu)
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Text menus for inserting inline content
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(menu-bind text-inline-menu
   (if (style-has? "std-markup-dtd")
       (-> "Content tag" (link content-tag-menu))
       (-> "Size tag" (link size-tag-menu)))
   (-> "Presentation tag" (link presentation-tag-menu))
   (if (style-has? "std-markup-dtd")
       (-> "Language" (link text-language-menu)))
-  (-> "Scripts" (link local-supported-scripts-menu))
+  (-> "Scripts" (link local-supported-scripts-menu)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Style dependent menus
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(menu-bind text-menu
+  (link text-block-menu)
+  (link text-extra-menu)
   ---
-  (-> "Mathematics" (link insert-math-menu))
-  (-> "Table" (link insert-table-menu))
-  (-> "Image" (link insert-image-menu))
-  (-> "Link" (link insert-link-menu))
-  (if (detailed-menus?)
-      (if (style-has? "std-fold-dtd")
-	  (-> "Fold" (link insert-fold-menu)))
-      (-> "Animation" (link insert-animation-menu)))
-  (if (and (style-has? "session-dtd") (detailed-menus?))
-      (-> "Session" (link insert-session-menu))))
+  (link text-inline-menu)
+  ---
+  (link texmacs-insert-menu))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Icons for inserting block markup
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(menu-bind text-block-icons
+  (if (style-has? "header-letter-dtd")
+      (=> (balloon (icon "tm_title.xpm") "Make a letter environment")
+          (link letter-header-menu)))
+  (if (style-has? "header-exam-dtd")
+      (=> (balloon (icon "tm_title.xpm") "Enter title information")
+          (link exam-header-menu)))
+  (if (style-has? "book-style")
+      (=> (balloon (icon "tm_chapter.xpm") "Start a new chapter")
+          (link chapter-menu)))
+  (if (and (style-has? "section-base-dtd")
+           (not (style-has? "header-exam-dtd"))
+           (not (in-poster?)))
+      (=> (balloon (icon "tm_section.xpm") "Start a new section")
+          (link section-menu)))
+  (if (in-poster?)
+      (=> (balloon (icon "tm_block.xpm") "Insert a section block")
+          (link poster-block-menu)))
+  (if (or (style-has? "env-theorem-dtd")
+          (style-has? "header-exam-dtd"))
+      (=> (balloon (icon "tm_theorem.xpm") "Insert an enunciation")
+          (link enunciation-menu)))
+  (if (and (style-has? "std-markup-dtd") (not (in-poster?)))
+      (=> (balloon (icon "tm_prominent.xpm") "Insert a prominent piece of text")
+          (link prominent-menu)))
+  (if (and (style-has? "std-markup-dtd") (in-poster?))
+      (=> (balloon (icon "tm_var_prominent.xpm")
+                   "Insert a prominent piece of text")
+          (link prominent-menu)))
+  (if (style-has? "std-markup-dtd")
+      (=> (balloon (icon "tm_program.xpm") "Insert a computer program")
+          (link code-menu)))
+  (if (style-has? "std-list-dtd")
+      (=> (balloon (icon "tm_list.xpm") "Insert a list")
+          (link list-menu)))
+  (if (and (style-has? "env-float-dtd") (detailed-menus?))
+      ;;((balloon (icon "tm_footnote.xpm") "Insert a footnote") ())
+      ;;((balloon (icon "tm_margin.xpm") "Insert a marginal note") ())
+      ;;((balloon (icon "tm_floating.xpm") "Insert a floating object") ())
+      ;;((balloon (icon "tm_multicol.xpm") "Start multicolumn context") ())
+      (=> (balloon (icon "tm_pageins.xpm") "Insert a note or a floating object")
+          (link note-menu)))
+  (if (style-has? "section-base-dtd")
+      (=> (balloon (icon "tm_index.xpm")
+                   "Insert automatically generated content")
+          (link automatic-menu))))
+
+(menu-bind text-extra-icons)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Icons for modifying text properties
@@ -461,6 +666,9 @@ for
   (=> (balloon (icon "tm_color.xpm") "Select a foreground color")
       (link color-menu)))
 
+(menu-bind text-inline-icons
+  (link text-format-icons))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Icons for text mode
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -468,48 +676,16 @@ for
 (menu-bind text-icons
   ;;("Goedenmiddag" (display* "Hi there\n"))
   ;;(mini #t (input (display* answer "\n") "string" '("Hello" "Bonjour") "0.5w"))
-  (if (style-has? "header-letter-dtd")
-      (=> (balloon (icon "tm_title.xpm") "Make a letter environment")
-	  (link letter-header-menu)))
-  (if (style-has? "header-exam-dtd")
-      (=> (balloon (icon "tm_title.xpm") "Enter title information")
-	  (link exam-header-menu)))
-  (if (style-has? "book-style")
-      (=> (balloon (icon "tm_chapter.xpm") "Start a new chapter")
-	  (link chapter-menu)))
-  (if (and (style-has? "section-base-dtd")
-	   (not (style-has? "header-exam-dtd")))
-      (=> (balloon (icon "tm_section.xpm") "Start a new section")
-	  (link section-menu)))
-  (if (or (style-has? "env-theorem-dtd")
-          (style-has? "header-exam-dtd"))
-      (=> (balloon (icon "tm_theorem.xpm") "Insert an enunciation")
-	  (link enunciation-menu)))
-  (if (style-has? "std-markup-dtd")
-      (=> (balloon (icon "tm_prominent.xpm") "Insert a prominent piece of text")
-	  (link prominent-menu)))
-  (if (style-has? "std-markup-dtd")
-      (=> (balloon (icon "tm_program.xpm") "Insert a computer program")
-	  (link code-menu)))
-  (if (style-has? "std-list-dtd")
-      (=> (balloon (icon "tm_list.xpm") "Insert a list")
-	  (link list-menu)))
-  (if (and (style-has? "env-float-dtd") (detailed-menus?))
-      ;;((balloon (icon "tm_footnote.xpm") "Insert a footnote") ())
-      ;;((balloon (icon "tm_margin.xpm") "Insert a marginal note") ())
-      ;;((balloon (icon "tm_floating.xpm") "Insert a floating object") ())
-      ;;((balloon (icon "tm_multicol.xpm") "Start multicolumn context") ())
-      (=> (balloon (icon "tm_pageins.xpm") "Insert a note or a floating object")
-          (link note-menu)))
-  (if (style-has? "section-base-dtd")
-      (=> (balloon (icon "tm_index.xpm")
-		   "Insert automatically generated content")
-	  (link automatic-menu)))
+  (link text-block-icons)
+  (link text-extra-icons)
   (if (style-has? "std-markup-dtd")
       /)
-  (link text-format-icons)
+  (link text-inline-icons)
   (if (in-manual?) (link tmdoc-icons))
-  (link texmacs-insert-icons))
+  (link texmacs-insert-icons)
+  (if (and (in-presentation?) (not (visible-icon-bar? 0)))
+      /
+      (link dynamic-icons)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Focus menus for entering title information
@@ -541,12 +717,10 @@ for
   ("Subtitle" (make-doc-data-element 'doc-subtitle))
   ("Author" (make-doc-data-element 'doc-author))
   ("Date" (make-doc-data-element 'doc-date))
-  ("Today"
-   (begin (make-doc-data-element 'doc-date) (make-arity 'date 0)))
+  ("Today" (begin (make-doc-data-element 'doc-date) (make 'date 0)))
   ("Miscellanous" (make-doc-data-element 'doc-misc))
   ("Note" (make-doc-data-element 'doc-note))
-  ("TeXmacs notice" (begin (make-doc-data-element 'doc-note)
-                           (make 'with-TeXmacs-text))))
+  (-> "Cite TeXmacs" (link cite-texmacs-menu)))
 
 (tm-menu (focus-title-hidden-menu)
   ("Running title" (make-doc-data-element 'doc-running-title))
@@ -563,7 +737,7 @@ for
 (tm-menu (focus-title-icons)
   (assuming (doc-data-has-hidden?)
     ((check (balloon (icon "tm_show_hidden.xpm") "Show hidden") "v"
-	    (doc-data-deactivated?))
+            (doc-data-deactivated?))
      (doc-data-activate-toggle)))
   (mini #t
     (inert ("Title" (noop))))
@@ -571,7 +745,9 @@ for
       (link focus-title-menu)
       (-> "Hidden" (link focus-title-hidden-menu)))
   (=> (balloon (icon "tm_focus_prefs.xpm") "Title presentation options")
-      (link focus-title-option-menu)))
+      (link focus-title-option-menu))
+  (=> (balloon (icon "tm_like.xpm") "Cite TeXmacs")
+      (link cite-texmacs-menu)))
 
 (tm-menu (focus-ancestor-menu t)
   (:require (doc-title-context? t))
@@ -593,8 +769,8 @@ for
 
 (tm-menu (focus-tag-edit-menu l)
   (:require (and (in? l '(author-email author-homepage author-misc))
-		 (or (test-doc-title-clustering? "cluster-all")
-		     (test-doc-title-clustering? "cluster-by-affiliation"))))
+                 (or (test-doc-title-clustering? "cluster-all")
+                     (test-doc-title-clustering? "cluster-by-affiliation"))))
   (with l* (symbol-append l '-note)
     (dynamic (focus-tag-edit-menu l*))))
 
@@ -702,7 +878,7 @@ for
   (:require (previous-section))
   (mini #t
     (=> (eval (tm/section-get-title-string (previous-section)))
-	(link focus-section-menu))))
+        (link focus-section-menu))))
 
 (tm-menu (focus-extra-menu t)
   (:require (section-context? t))
@@ -712,10 +888,13 @@ for
 
 (tm-menu (focus-extra-icons t)
   (:require (section-context? t))
-  //
   (mini #t
     (=> (eval (tm/section-get-title-string t))
-	(link focus-section-menu))))
+        (link focus-section-menu))))
+
+(tm-define (child-proposals t i)
+  (:require (and (tree-in? t '(bibliography bibliography*)) (== i 1)))
+  (bib-standard-styles))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Focus menu for lists
@@ -723,7 +902,7 @@ for
 
 (tm-define (standard-options l)
   (:require (in? l (list-tag-list)))
-  (list "compact-list"))
+  (list "compact-list" "triangle-list"))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Focus menu for theorems and proofs
@@ -733,9 +912,13 @@ for
   (:require (or (in? l (enunciation-tag-list))
                 (in? l (render-enunciation-tag-list))
                 (in? l '(proof render-proof))))
-  (list "number-europe" ;; "number-us"
-        "number-long-article"
-        "framed-theorems" "hanging-theorems"))
+  (append (list "number-europe" ;; "number-us"
+                "number-long-article"
+                "framed-theorems"
+                "hanging-theorems")
+          (if (style-has? "base-deco-dtd")
+              (list "shadowed-frames")
+              (list))))
 
 (tm-menu (focus-extra-menu t)
   (:require (dueto-supporting-context? t))
@@ -773,13 +956,13 @@ for
   (:require (algorithm-context? t))
   (when (not (algorithm-named? (focus-tree)))
     ((check (balloon (icon "tm_numbered.xpm") "Toggle numbering") "v"
-	    (algorithm-numbered? (focus-tree)))
+            (algorithm-numbered? (focus-tree)))
      (algorithm-toggle-number (focus-tree))))
   ((check (balloon (icon "tm_small_textual.xpm") "Toggle name") "v"
-	  (algorithm-named? (focus-tree)))
+          (algorithm-named? (focus-tree)))
    (algorithm-toggle-name t))
   ((check (balloon (icon "tm_specified.xpm") "Toggle specification") "v"
-	  (algorithm-specified? (focus-tree)))
+          (algorithm-specified? (focus-tree)))
    (algorithm-toggle-specification t)))
 
 (tm-define (standard-options l)
@@ -790,11 +973,78 @@ for
 ;; Focus menus for floating objects
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(tm-menu (focus-position-float-icons t)
-  (:require (float-context? t))
+(tm-menu (focus-float-icons t)
+  (:require (marginal-note-context? t))
+  (=> (balloon (icon "tm_position_float.xpm")
+               "Position of marginal note")
+      (link position-marginal-note-menu)))
+
+(tm-menu (focus-float-icons t)
+  (:require (rich-float-context? t))
+  (if (in-multicol-style?)
+      ((check (balloon (icon "tm_wide_float.xpm") "Make float wide") "v"
+              (float-wide? (focus-tree)))
+       (float-toggle-wide (focus-tree))))
   (=> (balloon (icon "tm_position_float.xpm")
                "Allowed positions of floating object")
-      (link position-float-menu)))
+      (link float-menu))
+  ((balloon (icon "tm_anchor.xpm")
+            "Go to anchor or float")
+   (cursor-toggle-anchor)))
+
+(tm-menu (focus-float-icons t)
+  (:require (phantom-float-context? t))
+  (=> (balloon (icon "tm_position_float.xpm")
+               "Allowed positions of floating object")
+      (link float-menu)))
+
+(tm-menu (focus-float-icons t)
+  (:require (floatable-context? t))
+  (if (in-multicol-style?)
+      ((check (balloon (icon "tm_wide_float.xpm") "Make wide") "v"
+              (floatable-wide? (focus-tree)))
+       (floatable-toggle-wide (focus-tree))))
+  ((balloon (icon "tm_position_float.xpm")
+            "Let the environment float")
+   (turn-floating (tree-innermost floatable-context?))))
+
+(tm-menu (focus-float-icons t)
+  (:require (footnote-context? t))
+  (if (in-multicol-style?)
+      ((check (balloon (icon "tm_wide_float.xpm") "Make footnote wide") "v"
+              (float-wide? (focus-tree)))
+       (float-toggle-wide (focus-tree))))
+  ((balloon (icon "tm_anchor.xpm")
+            "Go to anchor or footnote")
+   (cursor-toggle-anchor)))
+
+(tm-menu (focus-float-icons t)
+  (:require (balloon-context? t))
+  (=> (balloon (icon "tm_position_float.xpm")
+               "Alignment of balloon")
+      (link position-balloon-menu)))
+
+(tm-define (standard-options l)
+  (:require (in? l (numbered-unnumbered-append
+                     (append (small-figure-tag-list) (big-figure-tag-list)))))
+  (list "captions-above"))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Detached notes
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-menu (focus-toggle-menu t)
+  (:require (detached-note-context? t))
+  ((check "Named" "v" (custom-note-context? (focus-tree)))
+   (note-toggle-custom t))
+  (dynamic (former t)))
+
+(tm-menu (focus-toggle-icons t)
+  (:require (detached-note-context? t))
+  ((check (balloon (icon "tm_small_textual.xpm") "Use custom note symbol") "v"
+          (custom-note-context? (focus-tree)))
+   (note-toggle-custom t))
+  (dynamic (former t)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Possibility to rename titled environments
@@ -809,7 +1059,7 @@ for
 (tm-menu (focus-toggle-icons t)
   (:require (titled-context? t))
   ((check (balloon (icon "tm_small_textual.xpm") "Toggle name") "v"
-	  (titled-named? (focus-tree)))
+          (titled-named? (focus-tree)))
    (titled-toggle-name t))
   (dynamic (former t)))
 
@@ -826,6 +1076,21 @@ for
 (tm-menu (focus-toggle-icons t)
   (:require (or (frame-context? t) (frame-titled-context? t)))
   ((check (balloon (icon "tm_small_textual.xpm") "Toggle name") "v"
-	  (frame-titled? (focus-tree)))
+          (frame-titled? (focus-tree)))
    (frame-toggle-title t))
   (dynamic (former t)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Renaming automatically generated sections
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-menu (focus-toggle-menu t)
+  (:require (automatic-section-context? t))
+  (dynamic (former t))
+  ("Rename" (interactive automatic-section-rename)))
+
+(tm-menu (focus-toggle-icons t)
+  (:require (automatic-section-context? t))
+  (dynamic (former t))
+  ((balloon (icon "tm_small_textual.xpm") "Rename section")
+   (interactive automatic-section-rename)))
