@@ -13,18 +13,14 @@
 #include "merge_sort.hpp"
 #include "converter.hpp"
 #include "scheme.hpp"
+#include "ntuple.hpp"
 
 /******************************************************************************
-* Tests for caracters
+* Tests for characters
 ******************************************************************************/
 
 bool
-is_alpha (register char c) {
-  return ((c>='a') && (c<='z')) || ((c>='A') && (c<='Z'));
-}
-
-bool
-is_iso_alpha (register char c) {
+is_iso_alpha (char c) {
   int i= ((int) ((unsigned char) c));
   return
     ((c>='a') && (c<='z')) ||
@@ -33,7 +29,7 @@ is_iso_alpha (register char c) {
 }
 
 bool
-is_locase (register char c) {
+is_iso_locase (char c) {
   int code= (int) ((unsigned char) c);
   return
     ((c>='a') && (c<='z')) ||
@@ -42,7 +38,7 @@ is_locase (register char c) {
 }
 
 bool
-is_upcase (register char c) {
+is_iso_upcase (char c) {
   int code= (int) ((unsigned char) c);
   return
     ((c>='A') && (c<='Z')) ||
@@ -50,28 +46,6 @@ is_upcase (register char c) {
     ((code >= 192) && (code < 224));
 }
 
-bool
-is_digit (register char c) {
-  return (c>='0') && (c<='9');
-}
-
-bool
-is_numeric (register char c) {
-  return ((c>='0') && (c<='9')) || (c=='.');
-}
-
-bool
-is_punctuation (register char c) {
-  return
-    (c=='.') || (c==',') || (c==':') || (c=='\'') || (c=='`') ||
-    (c==';') || (c=='!') || (c=='?');
-}
-
-bool
-is_space (register char c) {
-  return (c == ' ') || (c == '\11') || (c == '\12') || (c == '\15');
-}
-
 /******************************************************************************
 * Tests for strings
 ******************************************************************************/
@@ -118,14 +92,14 @@ is_numeric (string s) {
 
 char
 upcase (char c) {
-  if (is_locase (c))
+  if (is_iso_locase (c))
     return (char) (((int) ((unsigned char) c)) - 32);
   else return c;
 }
 
 char
 locase (char c) {
-  if (is_upcase (c))
+  if (is_iso_upcase (c))
     return (char) (((int) ((unsigned char) c)) + 32);
   else return c;
 }
@@ -140,13 +114,13 @@ closing_delimiter (char c) {
 
 string
 upcase_first (string s) {
-  if ((N(s)==0) || (!is_locase (s[0]))) return s;
+  if ((N(s)==0) || (!is_iso_locase (s[0]))) return s;
   return string ((char) (((int) ((unsigned char) s[0]))-32)) * s (1, N(s));
 }
 
 string
 locase_first (string s) {
-  if ((N(s)==0) || (!is_upcase (s[0]))) return s;
+  if ((N(s)==0) || (!is_iso_upcase (s[0]))) return s;
   return string ((char) (((int) ((unsigned char) s[0]))+32)) * s (1, N(s));
 }
 
@@ -155,7 +129,7 @@ upcase_all (string s) {
   int i;
   string r (N(s));
   for (i=0; i<N(s); i++)
-    if (!is_locase (s[i])) r[i]= s[i];
+    if (!is_iso_locase (s[i])) r[i]= s[i];
     else r[i]= (char) (((int) ((unsigned char) s[i]))-32);
   return r;
 }
@@ -165,7 +139,7 @@ locase_all (string s) {
   int i;
   string r (N(s));
   for (i=0; i<N(s); i++)
-    if (!is_upcase (s[i])) r[i]= s[i];
+    if (!is_iso_upcase (s[i])) r[i]= s[i];
     else r[i]= (char) (((int) ((unsigned char) s[i]))+32);
   return r;
 }
@@ -202,20 +176,20 @@ ispanish_to_spanish (string s) {
   for (i=0; i<n; i++)
     if ((s[i] == '\'') && ((i+1)<n)) {
       switch (s[i+1]) {
-      case 'A': r << ''; break;
-      case 'E': r << ''; break;
-      case 'I': r << ''; break;
-      case 'N': r << ''; break;
-      case 'O': r << ''; break;
-      case 'U': r << ''; break;
-      case 'Y': r << ''; break;
-      case 'a': r << ''; break;
-      case 'e': r << ''; break;
-      case 'i': r << ''; break;
-      case 'n': r << ''; break;
-      case 'o': r << ''; break;
-      case 'u': r << ''; break;
-      case 'y': r << ''; break;
+      case 'A': r << '\301'; break;
+      case 'E': r << '\311'; break;
+      case 'I': r << '\315'; break;
+      case 'N': r << '\321'; break;
+      case 'O': r << '\323'; break;
+      case 'U': r << '\332'; break;
+      case 'Y': r << '\335'; break;
+      case 'a': r << '\341'; break;
+      case 'e': r << '\351'; break;
+      case 'i': r << '\355'; break;
+      case 'n': r << '\361'; break;
+      case 'o': r << '\363'; break;
+      case 'u': r << '\372'; break;
+      case 'y': r << '\375'; break;
       default : r << '\'' << s[i+1];
       }
       i++;
@@ -230,20 +204,20 @@ spanish_to_ispanish (string s) {
   string r;
   for (i=0; i<n; i++)
     switch (s[i]) {
-    case '': r << "'A"; break;
-    case '': r << "'E"; break;
-    case '': r << "'I"; break;
-    case '': r << "'N"; break;
-    case '': r << "'O"; break;
-    case '': r << "'U"; break;
-    case '': r << "'Y"; break;
-    case '': r << "'a"; break;
-    case '': r << "'e"; break;
-    case '': r << "'i"; break;
-    case '': r << "'n"; break;
-    case '': r << "'o"; break;
-    case '': r << "'u"; break;
-    case '': r << "'y"; break;
+    case '\301': r << "'A"; break;
+    case '\311': r << "'E"; break;
+    case '\315': r << "'I"; break;
+    case '\321': r << "'N"; break;
+    case '\323': r << "'O"; break;
+    case '\332': r << "'U"; break;
+    case '\335': r << "'Y"; break;
+    case '\341': r << "'a"; break;
+    case '\351': r << "'e"; break;
+    case '\355': r << "'i"; break;
+    case '\361': r << "'n"; break;
+    case '\363': r << "'o"; break;
+    case '\372': r << "'u"; break;
+    case '\375': r << "'y"; break;
     default : r << s[i];
     }
   return r;
@@ -254,7 +228,7 @@ igerman_to_german (string s) {
   int i, n= N(s);
   string r;
   for (i=0; i<n; i++)
-    if (s[i] == '') r << '';
+    if (s[i] == '\337') r << '\377';
     else r << s[i];
   return r;
 }
@@ -264,7 +238,7 @@ german_to_igerman (string s) {
   int i, n= N(s);
   string r;
   for (i=0; i<n; i++)
-    if (s[i] == '') r << '';
+    if (s[i] == '\377') r << '\337';
     else r << s[i];
   return r;
 }
@@ -274,9 +248,9 @@ german_to_igerman (string s) {
 ******************************************************************************/
 
 static string il2_to_cork_string=
-  "   ĈǃɆ˅΄ЋԎ.ږݕ䨢/ ";
+  "\200\201\202\203\204\205\206\207\210\211\212\213\214\215\216\217\220\221\222\223\224\225\226\227\230\231\232\233\234\235\236\237 \20\212 \211\221\237¨\222\223\224\231‐\232\233 \241˛\252´\251\261ˇ¸\262\263\264\271˝\272\273\217\301\302\200\304\210\202\307\203\311\206\313\205\315\316\204\320\213\214\323\324\216\326.\220\227\332\226\334\335\225\377\257\341\342\240\344\250\242\347\243\351\246\353\245\355\356\244\236\253\254\363\364\256\366/\260\267\372\266\374\375\265 ";
 static string cork_to_il2_string=
-  "áGť ئYIIg嵳 yi!?LAAAAEEIINOOOOUU Saaaaeeiinoooouu ";
+  "\303\241\306\310\317\314\312G\305\245\243\321\322 \325\300\330\246\251\252\253\336\333\331Y\254\256\257II\360\247\343\261\346\350\357\354\352g\345\265\263\361\362 \365\340\370\266\271\272\273\376\373\371y\274\276\277i!?LA\301\302A\304AA\307E\311E\313I\315\316I\320NO\323\324O\326OOU\332U\334\335 Sa\341\342a\344aa\347e\351e\353i\355\356i\360no\363\364o\366oou\372u\374\375 \337";
 
 static char
 il2_to_cork (char c) {
@@ -315,27 +289,27 @@ cork_to_il2 (string s) {
 ******************************************************************************/
 
 static string koi8_to_iso_string=
-  "";
+  "\341\342\367\347\344\345\366\372\351\352\353\354\355\356\357\360\362\363\364\365\346\350\343\376\373\375\377\371\370\374\340\361\301\302\327\307\304\305\326\332\311\312\313\314\315\316\317\320\322\323\324\325\306\310\303\336\333\335\337\331\330\334\300\321";
 static string iso_to_koi8_string=
-  "";
+  "\376\340\341\366\344\345\364\343\365\350\351\352\353\354\355\356\357\377\360\361\362\363\346\342\374\373\347\370\375\371\367\372\336\300\301\326\304\305\324\303\325\310\311\312\313\314\315\316\317\337\320\321\322\323\306\302\334\333\307\330\335\331\327\332";
 
 static char
 koi8_to_iso (char c, bool ukrainian) {
   int i= (int) ((unsigned char) c);
-  if (i==156) return '';
-  if (i==188) return '';
+  if (i==156) return '\263';
+  if (i==188) return '\243';
   if (ukrainian)
   {
      switch(c)
      {
-         case 'I':return '';
-         case '':return '';
-         case '':return '';
-         case '':return '';
-         case 'i':return '';
-         case '':return '';
-         case '':return '';
-         case '':return '';
+         case 'I':return '\266';
+         case '\210':return '\267';
+         case '\231':return '\264';
+         case '\200':return '\275';
+         case 'i':return '\246';
+         case '\250':return '\247';
+         case '\271':return '\244';
+         case '\240':return '\255';
      }
   }
   if (i<192) return c;
@@ -345,20 +319,20 @@ koi8_to_iso (char c, bool ukrainian) {
 static char
 iso_to_koi8 (char c, bool ukrainian) {
   int i= (int) ((unsigned char) c);
-  if (c=='') return (char) 156;
-  if (c=='') return (char) 188;
+  if (c=='\263') return (char) 156;
+  if (c=='\243') return (char) 188;
   if (ukrainian)
   {
      switch(c)
      {
-         case '':return 'I';
-         case '':return '';
-         case '':return '';
-         case '':return '';
-         case '':return 'i';
-         case '':return '';
-         case '':return '';
-         case '':return '';
+         case '\266':return 'I';
+         case '\267':return '\210';
+         case '\264':return '\231';
+         case '\275':return '\200';
+         case '\246':return 'i';
+         case '\247':return '\250';
+         case '\244':return '\271';
+         case '\255':return '\240';
      }
   }
   if (i<192) return c;
@@ -440,121 +414,6 @@ convert_OTS1_symbols_to_universal_encoding (tree t) {
   return r;
 }
 
-/******************************************************************************
-* Convert between TeXmacs and XML strings
-******************************************************************************/
-
-static bool
-is_xml_name (char c) {
-  return
-    is_alpha (c) || is_numeric (c) ||
-    (c == '.') || (c == '-') || (c == ':');
-}
-
-string
-tm_to_xml_name (string s) {
-  string r;
-  int i, n= N(s);
-  for (i=0; i<n; i++)
-    if (is_xml_name (s[i])) r << s[i];
-    else r << "_" << as_string ((int) ((unsigned char) s[i])) << "_";
-  return r;
-}
-
-string
-xml_name_to_tm (string s) {
-  string r;
-  int i, n= N(s);
-  for (i=0; i<n; i++)
-    if (s[i] != '_') r << s[i];
-    else {
-      int start= ++i;
-      while ((i<n) && (s[i]!='_')) i++;
-      r << (char) ((unsigned char) as_int (s (start, i)));
-    }
-  return r;
-}
-
-string
-old_tm_to_xml_cdata (string s) {
-  string r;
-  int i, n= N(s);
-  for (i=0; i<n; i++)
-    if (s[i] == '&') r << "&amp;";
-    else if (s[i] == '>') r << "&gt;";
-    else if (s[i] != '<') r << s[i];
-    else {
-      int start= ++i;
-      while ((i<n) && (s[i]!='>')) i++;
-      r << "&" << tm_to_xml_name (s (start, i)) << ";";
-    }
-  return r;
-}
-
-object
-tm_to_xml_cdata (string s) {
-  array<object> a;
-  a << symbol_object ("!concat");
-  string r;
-  int i, n= N(s);
-  for (i=0; i<n; i++)
-    if (s[i] == '&') r << "&amp;";
-    else if (s[i] == '>') r << "&gt;";
-    else if (s[i] == '\\') r << "\\";
-    else if (s[i] != '<') r << cork_to_utf8 (s (i, i+1));
-    else {
-      int start= i++;
-      while ((i<n) && (s[i]!='>')) i++;
-      string ss= s (start, i+1);
-      string rr= cork_to_utf8 (ss);
-      string qq= utf8_to_cork (rr);
-      if (rr != ss && qq == ss) r << rr;
-      else {
-	if (r != "") a << object (r);
-	a << cons (symbol_object ("tm-sym"),
-		   cons (ss (1, N(ss)-1),
-			 null_object ()));
-	r= "";
-      }
-    }
-  if (r != "") a << object (r);
-  if (N(a) == 1) return object ("");
-  else if (N(a) == 2) return a[1];
-  else return call ("list", a);
-}
-
-string
-old_xml_cdata_to_tm (string s) {
-  string r;
-  int i, n= N(s);
-  for (i=0; i<n; i++)
-    if (s[i] == '<') r << "<less>";
-    else if (s[i] == '>') r << "<gtr>";
-    else if (s[i] != '&') r << s[i];
-    else {
-      int start= ++i;
-      while ((i<n) && (s[i]!=';')) i++;
-      string x= "<" * xml_name_to_tm (s (start, i)) * ">";
-      if (x == "<amp>") r << "&";
-      else r << x;
-    }
-  return r;
-}
-
-string
-xml_unspace (string s, bool first, bool last) {
-  string r;
-  int i= 0, n= N(s);
-  if (first) while ((i<n) && is_space (s[i])) i++;
-  while (i<n)
-    if (!is_space (s[i])) r << s[i++];
-    else {
-      while ((i<n) && is_space (s[i])) i++;
-      if ((i<n) || (!last)) r << ' ';
-    }
-  return r;
-}
-
 bool
 contains_unicode_char (string s) {
   int i= 0, n= N(s);
@@ -658,7 +517,7 @@ from_hexadecimal (string s) {
     return -from_hexadecimal (s (1, n));
   for (i=0; i<n; i++) {
     res= res << 4;
-    if ((s[i] >= '0') && (s[i] <= '9')) res += (int) (s[i] - '0');
+    if (is_digit (s[i])) res += (int) (s[i] - '0');
     if ((s[i] >= 'A') && (s[i] <= 'F')) res += (int) (s[i] + 10 - 'A');
     if ((s[i] >= 'a') && (s[i] <= 'f')) res += (int) (s[i] + 10 - 'a');
   }
@@ -672,7 +531,7 @@ from_hexadecimal (string s) {
 string
 tm_encode (string s) {
   // verbatim to TeXmacs encoding
-  register int i;
+  int i;
   string r;
   for (i=0; i<N(s); i++) {
     if (s[i]=='<') r << "<less>";
@@ -685,16 +544,18 @@ tm_encode (string s) {
 string
 tm_decode (string s) {
   // TeXmacs encoding to verbatim
-  register int i;
+  int i;
   string r;
   for (i=0; i<N(s); i++) {
     if (s[i]=='<') {
-      register int j;
+      int j;
       for (j=i+1; j<N(s); j++)
-	if (s[j]=='>') break;
+        if (s[j]=='>') break;
       if (j<N(s)) j++;
       if (s(i,j) == "<less>") r << "<";
       else if (s(i,j) == "<gtr>") r << ">";
+      else if (i+7==j && s[i+1]=='#' && s[j-1]=='>')
+        r << s(i, j);
       i=j-1;
       if (s[i]!='>') return r;
     }
@@ -705,13 +566,13 @@ tm_decode (string s) {
 
 string
 tm_var_encode (string s) {
-  register int i, n= N(s);
+  int i, n= N(s);
   string r;
   for (i=0; i<n; i++) {
     if (s[i]=='<') {
       if (i+1 < n && s[i+1] == '#') {
-	while (i<n && s[i] != '>') r << s[i++];
-	if (i<n) r << s[i];
+        while (i<n && s[i] != '>') r << s[i++];
+        if (i<n) r << s[i];
       }
       else r << "<less>";
     }
@@ -723,17 +584,17 @@ tm_var_encode (string s) {
 
 string
 tm_correct (string s) {
-  register int i;
+  int i;
   string r;
   for (i=0; i<N(s); i++) {
     if (s[i]=='<') {
-      register bool flag= true;
-      register int j, k;
+      bool flag= true;
+      int j, k;
       for (j=i+1; j<N(s); j++)
-	if (s[j]=='>') break;
+        if (s[j]=='>') break;
       if (j==N(s)) return r;
       for (k=i+1; k<j; k++)
-	if (s[k]=='<') flag= false;
+        if (s[k]=='<') flag= false;
       if (flag) r << s(i,j+1);
       i=j;
     }
@@ -929,7 +790,7 @@ scm_quote (string s) {
 
 string
 scm_unquote (string s) {
-  if ((N(s)>=2) && (s[0]=='\"') && (s[N(s)-1]=='\"')) {
+  if (is_quoted (s)) {
     int i, n= N(s);
     string r;
     for (i=1; i<n-1; i++)
@@ -950,7 +811,7 @@ raw_quote (string s) {
 string
 raw_unquote (string s) {
   // Get the string value of a STRING tree label representing a string.
-  if ((N(s)>=2) && (s[0]=='\"') && (s[N(s)-1]=='\"'))
+  if (is_quoted (s))
     return s (1, N(s)-1);
   else return s;
 }
@@ -961,7 +822,7 @@ raw_unquote (string s) {
 
 string
 escape_sh (string s) {
-#if defined (__MINGW__) || defined (__MINGW32__) || defined (OS_WIN32)
+#ifdef OS_MINGW
   return raw_quote (s);
 #else
   int i, n= N(s);
@@ -1042,7 +903,11 @@ unescape_guile (string s) {
   string r;
   for (i=0; i<n; i++) {
     if (s[i] == '\\') {
-      if (i+3 < n && s[i+1] == 'x'
+      if (i+1 < n && s[i+1] == '\\') {
+        r << "\\\\\\\\";
+        i+=1;
+      }
+      else if (i+3 < n && s[i+1] == 'x'
           && is_hex_digit (s[i+2]) && is_hex_digit (s[i+3])) {
         string e= s(i+2, i+4);
         r << (unsigned char) from_hexadecimal (e);
@@ -1189,6 +1054,13 @@ read_double (string s, int& i, double& result) {
   return true;
 }
 
+bool
+is_whitespace (string s) {
+  for (int i=0; i<N(s); i++)
+    if (s[i] != ' ' && s[i] != '\t' && s[i] != '\n') return false;
+  return true;
+}
+
 void
 skip_spaces (string s, int& i) {
   int n=N(s);
@@ -1214,7 +1086,7 @@ skip_symbol (string s, int& i) {
   if (i<n) {
     if (s[i]=='<') {
       for (i++; i<n; i++)
-	if (s[i-1]=='>') break;
+        if (s[i-1]=='>') break;
     }
     else i++;
   }
@@ -1240,6 +1112,27 @@ convert_tabs_to_spaces (string s, int tw) {
   return r;
 }
 
+string
+downgrade_math_letters (string s) {
+  string r= "";
+  for (int i=0; i<N(s); ) {
+    int start= i;
+    tm_char_forwards (s, i);
+    if (i == start + 1) r << s[start];
+    else {
+      string ss= s (start, i);
+      if (starts (ss, "<b-")) ss= "<" * ss (3, N(ss));
+      if (starts (ss, "<up-")) ss= "<" * ss (4, N(ss));
+      if (starts (ss, "<cal-")) ss= "<" * ss (5, N(ss));
+      if (starts (ss, "<bbb-")) ss= "<" * ss (5, N(ss));
+      if (starts (ss, "<frak-")) ss= "<" * ss (6, N(ss));
+      if (N(ss) == 3) ss= ss (1, 2);
+      r << ss;
+    }
+  }
+  return r;
+}
+
 /******************************************************************************
 * Parsing binary data
 ******************************************************************************/
@@ -1637,3 +1530,111 @@ strip_completions (array<string> a, string prefix) {
       b << a[i] (N(prefix), N(a[i]));
   return b;
 }
+
+/******************************************************************************
+* Differences between two strings
+******************************************************************************/
+
+static int
+find_longest (string s1, string s2, int& c1, int& c2) {
+  int n1= N(s1), n2= N(s2), bc= 0, bl= 0, br= 0;
+  for (c2=0; c2<n2; c2++)
+    if (s1[c1] == s2[c2]) {
+      int l=0, r=0;
+      while (c1+r<n1 && c2+r<n2 && s1[c1+r] == s2[c2+r]) r++;
+      while (l<c1 && l<c2 && s1[c1-l-1] == s2[c2-l-1]) l++;
+      if (l+r > bl+br) {
+        bc= c2;
+        bl= l;
+        br= r;
+      }
+    }
+  if (bl + br > 0) {
+    c1= c1 - bl;
+    c2= bc - bl;
+  }
+  return bl + br;
+}
+
+static void
+find_common (string s1, string s2, int& c1, int& c2) {
+  int best_len= 0;
+  c1= c2= 0;
+  int n1= N(s1), n2= N(s2);
+  if (n1 == 0 || n2 == 0) return;
+  int t= min (min (n1, n2), 6);
+  for (int k=1; k<t; k++) {
+    int a1= (k*n1)/t, a2= (k*n2)/t;
+    int len= find_longest (s1, s2, a1, a2);
+    if (len > best_len) { best_len= len; c1= a1; c2= a2; }
+  }
+}
+
+array<int>
+differences (string s1, string s2) {
+  int n1= N(s1), n2= N(s2);
+  int i1= 0, i2= 0, j1= n1, j2= n2;
+  while (i1<j1 && i2<j2 && s1[i1] == s2[i2]) { i1++; i2++; }
+  while (i1<j1 && i2<j2 && s1[j1-1] == s2[j2-1]) { j1--; j2--; }
+  if (i1 == i2 && j1 == j2) return array<int> ();
+  if (i1 > 0 || i2 > 0 || j1 < n1 || j2 < n2) {
+    array<int> r= differences (s1 (i1, j1), s2 (i2, j2));
+    for (int k=0; k<N(r); k+=4) {
+      r[k  ] += i1;
+      r[k+1] += i1;
+      r[k+2] += i2;
+      r[k+3] += i2;
+    }
+    return r;
+  }
+  else {
+    int c1, c2;
+    find_common (s1, s2, c1, c2);
+    if (c1 == 0 && c2 == 0) {
+      array<int> r;
+      r << i1 << j1 << i2 << j2;
+      return r;
+    }
+    else {
+      array<int> r1= differences (s1 (0 , c1), s2 (0 , c2));
+      array<int> r2= differences (s1 (c1, n1), s2 (c2, n2));
+      for (int k=0; k<N(r2); k+=4) {
+        r2[k  ] += c1;
+        r2[k+1] += c1;
+        r2[k+2] += c2;
+        r2[k+3] += c2;
+      }
+      r1 << r2;
+      return r1;
+    }
+  }
+}
+
+int
+distance (string s1, string s2) {
+  int d= 0;
+  array<int> r= differences (s1, s2);
+  for (int k=0; k<N(r); k+=4)
+    d += max (r[k+1] - r[k], r[k+3] - r[k+2]);
+  return d;
+}
+
+/******************************************************************************
+* Parse length
+******************************************************************************/
+
+void
+parse_length (string s, double& len, string& unit) {
+  int start= 0;
+  int i, n= N(s);
+  for (i=start; i<n && !is_locase (s[i]); i++) {}
+  string s1= s (start, i);
+  string s2= s (i, n);
+  if (is_double (s1) && (is_locase_alpha (s2) || is_empty (s2))) {
+    len= as_double (s1);
+    unit= s2;
+  } else {
+    len= 0.0;
+    unit= "error";
+  }
+}
