@@ -154,7 +154,12 @@ get_env_child (tree t, int i, tree env) {
 
 tree
 get_env_descendant (tree t, path p, tree env) {
-  return the_drd->get_env_descendant (t, p, env);  
+  return the_drd->get_env_descendant (t, p, env);
+}
+
+tree
+get_env_descendant (tree t, path p, string var, tree val) {
+  return the_drd->get_env_descendant (t, p, var, val);
 }
 
 /******************************************************************************
@@ -172,17 +177,21 @@ move_any (tree t, path p, bool forward) {
   }
   if (is_atomic (st)) {
     string s= st->label;
+#ifdef SANITY_CHECKS
     ASSERT (l >= 0 && l <= N(s), "out of range");
+#else
+    l= max (min (l, N(s)), 0);
+#endif
     if (forward) {
       if (l<N(s)) {
-	tm_char_forwards (s, l);
-	return q * l;
+        tm_char_forwards (s, l);
+        return q * l;
       }
     }
     else {
       if (l>0) {
-	tm_char_backwards (s, l);
-	return q * l;
+        tm_char_backwards (s, l);
+        return q * l;
       }
     }
   }
@@ -190,13 +199,13 @@ move_any (tree t, path p, bool forward) {
     int i, n= N(st);
     if (forward) {
       for (i=0; i<n; i++)
-	if (the_drd->is_accessible_child (st, i))
-	  return q * path (i, 0);
+        if (the_drd->is_accessible_child (st, i))
+          return q * path (i, 0);
     }
     else {
       for (i=n-1; i>=0; i--)
-	if (the_drd->is_accessible_child (st, i))
-	  return q * path (i, right_index (st[i]));
+        if (the_drd->is_accessible_child (st, i))
+          return q * path (i, right_index (st[i]));
     }
     return q * (1-l);
   }
@@ -209,12 +218,12 @@ move_any (tree t, path p, bool forward) {
   if (forward) {
     for (i=l+1; i<n; i++)
       if (the_drd->is_accessible_child (st, i))
-	return q * path (i, 0);
+        return q * path (i, 0);
   }
   else {
     for (i=l-1; i>=0; i--)
       if (the_drd->is_accessible_child (st, i)) {
-	return q * path (i, right_index (st[i]));
+        return q * path (i, right_index (st[i]));
     }
   }
   return q * (forward? 1: 0);
@@ -229,9 +238,36 @@ path previous_any (tree t, path p) {
 * Traversal of a valid cursor positions inside a tree
 ******************************************************************************/
 
+path
+closest_up (tree t, path p) {
+  if (is_atomic (t)) {
+    if (is_nil (p)) return path (0);
+    else return path (max (0, min (p->item, N(t->label))));
+  }
+  else if (is_concat (t) || is_document (t)) {
+    if (N(t) == 0) return path (0);
+    else if (is_nil (p) || is_atom (p) || p->item < 0)
+      return path (0, closest_up (t[0], path ()));
+    else if (p->item >= N(t))
+      return path (N(t)-1, closest_up (t[N(t)-1], path ()));
+    else return path (p->item, closest_up (t[p->item], p->next));
+  }
+  else {
+    if (is_nil (p)) return path (0);
+    else if (is_atom (p)) return path (max (0, min (p->item, 1)));
+    else if (p->item < 0) return path (0);
+    else if (p->item >= N(t)) return path (1);
+    else return path (p->item, closest_up (t[p->item], p->next));
+  }
+}
+
 static path
 move_valid_sub (tree t, path p, bool forward) {
-  ASSERT (is_inside (t, p), "invalid cursor");
+#ifdef SANITY_CHECKS
+  ASSERT (is_inside (t, p), "invalid cursor [move_valid]");
+#else
+  if (!is_inside (t, p)) p= closest_up (t, p);
+#endif
   path q= p;
   while (true) {
     path r= move_any (t, q, forward);
@@ -258,7 +294,11 @@ path previous_valid (tree t, path p) {
 
 static path
 move_accessible (tree t, path p, bool forward) {
-  ASSERT (is_inside (t, p), "invalid cursor");
+#ifdef SANITY_CHECKS
+  ASSERT (is_inside (t, p), "invalid cursor [move_accessible]");
+#else
+  if (!is_inside (t, p)) p= closest_up (t, p);
+#endif
   path q= p;
   while (true) {
     path r= move_any (t, q, forward);
@@ -312,23 +352,23 @@ move_word (tree t, path p, bool forward) {
       int n= N(s);
       if (s == "") return q;
       if (forward && l>0 &&
-	  (is_iso_alphanum (s[l-1]) || (l<n && s[l] == ' ') ||
-	   (l==n && at_border (t, path_up (q), forward))) &&
-	  (l==n || !is_iso_alphanum (s[l])))
-	return q;
+          (is_iso_alphanum (s[l-1]) || (l<n && s[l] == ' ') ||
+           (l==n && at_border (t, path_up (q), forward))) &&
+          (l==n || !is_iso_alphanum (s[l])))
+        return q;
       if (!forward && l<n &&
-	  (is_iso_alphanum (s[l]) || (l>0 && s[l-1] == ' ') ||
-	   (l==0 && at_border (t, path_up (q), forward))) &&
-	  (l==0 || !is_iso_alphanum (s[l-1])))
-	return q;
+          (is_iso_alphanum (s[l]) || (l>0 && s[l-1] == ' ') ||
+           (l==0 && at_border (t, path_up (q), forward))) &&
+          (l==0 || !is_iso_alphanum (s[l-1])))
+        return q;
       if (!forward && l==n && next_is_word (t, path_up (q)))
-	return q;
+        return q;
     }
     else {
       if (forward && l==1) return q;
       if (!forward && l==0) return q;
       if (!forward && next_is_word (t, path_up (q)))
-	return q;
+        return q;
     }
     p= q;
   }
@@ -350,7 +390,7 @@ move_node (tree t, path p, bool forward) {
     if (forward) p= path_up (p) * N (st->label);
     else p= path_up (p) * 0;
   }
-  p= move_valid (t, p, forward);
+  p= move_accessible (t, p, forward);
   st= subtree (t, path_up (p));
   if (is_atomic (st) && last_item (p) > 0)
     p= path_up (p) * N (st->label);
@@ -384,7 +424,7 @@ distinct_tag_or_argument (tree t, path p, path q, hashset<int> labs) {
   while (!is_nil (r) && (r != c)) {
     r= path_up (r);
     if (labs->contains ((int) L (subtree (t, r))) &&
-	!none_accessible (subtree (t, r)))
+        !none_accessible (subtree (t, r)))
       return true;
   }
   return false;
@@ -435,7 +475,7 @@ move_tag (tree t, path p, hashset<int> labs, bool forward, bool preserve) {
     if (distinct_tag_or_argument (t, p, r, labs) &&
         acceptable_border (t, p, r, labs) &&
         acceptable_child (t, r, labs) &&
-	(!preserve || tag_index (t, r, labs) == tag_index (t, p, labs)))
+        (!preserve || tag_index (t, r, labs) == tag_index (t, p, labs)))
       return r;
     q= r;
   }
@@ -450,7 +490,7 @@ get_labels (scheme_tree t) {
     int i, n= N(t);
     for (i=0; i<n; i++)
       if (is_atomic (t[i]))
-	labs->insert ((int) as_tree_label (t[i]->label));
+        labs->insert ((int) as_tree_label (t[i]->label));
   }
   return labs;
 }
@@ -478,12 +518,12 @@ move_argument (tree t, path p, bool forward) {
   if (forward) {
     for (i=l+1; i<n; i++)
       if (the_drd->is_accessible_child (st, i))
-	return start (t, q * i);
+        return start (t, q * i);
   }
   else {
     for (i=l-1; i>=0; i--)
       if (the_drd->is_accessible_child (st, i))
-	return end (t, q * i);
+        return end (t, q * i);
   }
   return path ();
 }
@@ -497,48 +537,64 @@ path previous_argument (tree t, path p) {
 * Other routines
 ******************************************************************************/
 
-static path
-search_upwards (tree t, path p, tree_label which) {
-  if (is_nil (p) || L (subtree (t, p)) == which) return p;
-  else return search_upwards (t, path_up (p), which);
+path
+search_common (tree t, path p, path q, tree_label which) {
+  if (is_atomic (t)) return path ();
+  if (is_nil (p) || is_nil (q) || p->item != q->item) return path ();
+  if (0 > p->item || p->item >= N(t)) return path ();
+  path c= search_common (t[p->item], p->next, q->next, which);
+  if (L(subtree (t[p->item], c)) == which) return path (p->item, c);
+  return path ();
+}
+
+bool
+no_other (tree t, path p, tree_label which) {
+  if (L(t) == which) return false;
+  if (is_nil (p)) return true;
+  if (0 > p->item || p->item >= N(t)) return false;
+  return no_other (t[p->item], p->next, which);
 }
 
 bool
 inside_same (tree t, path p, path q, tree_label which) {
-  return
-    search_upwards (t, path_up (p), which) ==
-    search_upwards (t, path_up (q), which);
+  path d= search_common (t, path_up (p), path_up (q), which);
+  if (L(subtree (t, d)) != which) return true;
+  path sp= p / d;
+  path sq= q / d;
+  if (is_atom (sp) || is_atom (sq)) return false;
+  if (sp->item != sq->item) return false;
+  if (0 > sp->item || sp->item >= N(subtree (t, d))) return false;
+  tree st= subtree (t, d) [sp->item];
+  return no_other (st, path_up (sq->next), which) ||
+         no_other (st, path_up (sp->next), which);
 }
 
 bool
-more_inside (tree t, path p, path q, tree_label which) {
-  return
-    search_upwards (t, path_up (q), which) <=
-    search_upwards (t, path_up (p), which);
+inside_same_or_more (tree t, path p, path q, tree_label which) {
+  path d= search_common (t, path_up (p), path_up (q), which);
+  if (L(subtree (t, d)) != which) return true;
+  path sp= p / d;
+  path sq= q / d;
+  if (is_atom (sp) || is_atom (sq)) return false;
+  if (sp->item != sq->item) return false;
+  if (0 > sp->item || sp->item >= N(subtree (t, d))) return false;
+  tree st= subtree (t, d) [sp->item];
+  return no_other (st, path_up (sq->next), which);
 }
 
 bool
-var_inside_same_sub (tree t, path p, path q, tree_label which) {
-  path pq= search_upwards (t, path_up (q), which);
-  path pp= search_upwards (t, path_up (p), which);
-  if (!(pq <= pp)) return false;
-  path c= common (p, q);
-  tree st= subtree (t, c);
-  path sp= p / c;
-  path sq= q / c;
-  if (N(sp) == 0 || N(sq) == 0) return false;
-  if (sp == path (0) || sp == path (1)) return true;
-  if (sq == path (0) || sq == path (1)) return true;
-  // TODO: maybe further adjustments will be necessary in the future
-  return false;
-}
-
-bool
-var_inside_same (tree t, path p, path q, tree_label which) {
-  return
-    inside_same (t, p, q, which) ||
-    var_inside_same_sub (t, p, q, which) ||
-    var_inside_same_sub (t, q, p, which);
+inside_contiguous_document (tree t, path op, path oq) {
+  if (!inside_same (t, op, oq, DOCUMENT)) return false;
+  path p= path_up (op), q= path_up (oq);
+  while (!is_nil (p) && !is_func (subtree (t, p), DOCUMENT)) p= path_up (p);
+  while (!is_nil (q) && !is_func (subtree (t, q), DOCUMENT)) q= path_up (q);
+  if (p == q) return true;
+  if (q <= p) return inside_contiguous_document (t, oq, op);
+  if (!(p <= q)) return false;
+  if (path_less (op, oq))
+    return (oq / q) == start (subtree (t, q));
+  else
+    return (oq / q) == end (subtree (t, q));
 }
 
 /******************************************************************************
@@ -577,12 +633,12 @@ previous_section_impl (tree t, path p) {
     int i= is_nil (p)? N(t)-1: p->item;
     for (; i>=0; i--) {
       if (!is_nil (p) && i == p->item) {
-	path r= previous_section_impl (t[i], p->next);
-	if (!is_nil (r)) return path (i, r);
+        path r= previous_section_impl (t[i], p->next);
+        if (!is_nil (r)) return path (i, r);
       }
       else {
-	path r= previous_section_impl (t[i], path ());
-	if (!is_nil (r)) return path (i, r);
+        path r= previous_section_impl (t[i], path ());
+        if (!is_nil (r)) return path (i, r);
       }
     }
   }
