@@ -15,7 +15,7 @@
 #include "convert.hpp"
 #include "file.hpp"
 #include "analyze.hpp"
-#include "timer.hpp"
+#include "tm_timer.hpp"
 #include "Bridge/impl_typesetter.hpp"
 #include "new_style.hpp"
 #include "iterator.hpp"
@@ -32,14 +32,18 @@ bool enable_fastenv= false;
 ******************************************************************************/
 
 edit_typeset_rep::edit_typeset_rep ():
+  editor_rep (), // NOTE: ignored by the compiler, but suppresses warning
   the_style (TUPLE),
   cur (hashmap<string,tree> (UNINIT)),
-  stydef (UNINIT), pre (UNINIT), init (UNINIT), fin (UNINIT),
+  stydef (UNINIT), pre (UNINIT), init (UNINIT), fin (UNINIT), grefs (UNINIT),
   env (drd, buf->buf->master,
-       buf->data->ref, (buf->prj==NULL? buf->data->ref: buf->prj->data->ref),
+       buf->data->ref, (buf->prj==NULL? grefs: buf->prj->data->ref),
        buf->data->aux, (buf->prj==NULL? buf->data->aux: buf->prj->data->aux),
        buf->data->att, (buf->prj==NULL? buf->data->att: buf->prj->data->att)),
-  ttt (new_typesetter (env, subtree (et, rp), reverse (rp))) {}
+  ttt (new_typesetter (env, subtree (et, rp), reverse (rp))) {
+    init_update ();
+}
+
 edit_typeset_rep::~edit_typeset_rep () { delete_typesetter (ttt); }
 
 void
@@ -207,6 +211,37 @@ edit_typeset_rep::typeset_prepare () {
   env->update ();
 }
 
+void
+edit_typeset_rep::init_update () {
+  if (buf->prj != NULL) {
+    string id= as_string (delta (buf->prj->buf->name, buf->buf->name));
+    string lab= "part:" * id;
+    hashmap<string,tree> aux= env->global_aux;
+    hashmap<string,tree> ref= env->global_ref;
+    if (aux->contains ("parts")) {
+      tree parts= aux ["parts"];
+      if (is_func (parts, DOCUMENT))
+        for (int i=0; i<N(parts); i++)
+          if (is_tuple (parts[i]) && N(parts[i]) >= 1)
+            if (parts[i][0] == id)
+              for (int j=1; j+1 < N(parts[i]); j+=2)
+                if (is_atomic (parts[i][j])) {
+                  buf->data->init (parts[i][j]->label)= copy (parts[i][j+1]);
+                  init (parts[i][j]->label)= copy (parts[i][j+1]);
+                }      
+    }
+    if (ref->contains (lab)) {
+      tree val= ref [lab];
+      if (is_tuple (val) && N(val) >= 2 && val[1] != tree (UNINIT)) {
+        buf->data->init (PAGE_FIRST)= copy (val[1]);
+        init (PAGE_FIRST)= copy (val[1]);
+      }
+    }
+  }
+  else if (buf->data->init ["part-flag"] == "true")
+    grefs= copy (buf->data->ref);
+}
+
 void
 edit_typeset_rep::drd_update () {
   typeset_exec_until (tp);
@@ -245,6 +280,12 @@ restricted_exec (edit_env env, tree t, int end) {
   else if (is_compound (t, "hide-preamble", 1) ||
            is_compound (t, "show-preamble", 1))
     env->exec (t[0]);
+  else if (is_compound (t, "script-input", 4) && end == 2) {
+    if (env->read (MODE) == "text") {
+      env->write (MODE, "prog");
+      env->write (PROG_LANGUAGE, t[0]);
+    }
+  }
 }
 
 static tree
@@ -329,10 +370,13 @@ edit_typeset_rep::typeset_exec_until (path p) {
         //cout << "t= " << t << "\n";
         //cout << "i= " << i << "\n";
         //cout << "w= " << w << "\n";
+        tree ww (w, N(w));
         for (int j=0; j<N(w); j+=2) {
           //cout << w[j] << " := " << env->exec (w[j+1]) << "\n";
-          env->write (w[j]->label, env->exec (w[j+1]));
+          ww[j+1]= env->exec (w[j+1]);
         }
+        for (int j=0; j<N(w); j+=2)
+          env->write (w[j]->label, ww[j+1]);
         t= t[i];
         q= q->next;
       }
@@ -396,10 +440,12 @@ tree
 edit_typeset_rep::get_init_value (string var) {
   if (init->contains (var)) {
     tree t= init [var];
+    if (var == BG_COLOR && is_func (t, PATTERN)) t= env->exec (t);
     return is_func (t, BACKUP, 2)? t[0]: t;
   }
   if (N(pre)==0) typeset_preamble ();
   tree t= pre [var];
+  if (var == BG_COLOR && is_func (t, PATTERN)) t= env->exec (t);
   return is_func (t, BACKUP, 2)? t[0]: t;
 }
 
@@ -443,18 +489,52 @@ edit_typeset_rep::get_env_language () {
   else return prog_language (get_env_string (PROG_LANGUAGE));
 }
 
-SI
-edit_typeset_rep::get_page_width () {
-  (void) get_env_string (PAGE_WIDTH);
-  return (env->page_user_width + env->page_odd_margin +
-          env->page_right_margin + std_shrinkf - 1) / std_shrinkf;
+int
+edit_typeset_rep::get_page_count () {
+  return N (eb[0]);
 }
 
 SI
-edit_typeset_rep::get_page_height () {
+edit_typeset_rep::get_page_width (bool deco) {
+  (void) get_env_string (PAGE_WIDTH);
+  return (env->get_page_width (deco) + std_shrinkf - 1) / std_shrinkf;
+}
+
+SI
+edit_typeset_rep::get_pages_width (bool deco) {
+  (void) get_env_string (PAGE_WIDTH);
+  return (env->get_pages_width (deco) + std_shrinkf - 1) / std_shrinkf;
+}
+
+SI
+edit_typeset_rep::get_page_height (bool deco) {
   (void) get_env_string (PAGE_HEIGHT);
-  return (env->page_user_height + env->page_top_margin +
-          env->page_bottom_margin + std_shrinkf - 1) / std_shrinkf;
+  return (env->get_page_height (deco) + std_shrinkf - 1) / std_shrinkf;
+}
+
+SI
+edit_typeset_rep::get_total_width (bool deco) {
+  SI w= eb->w();
+  if (!deco) {
+    SI w1= env->get_pages_width (false);
+    SI w2= env->get_pages_width (true);
+    w -= (w2 - w1);
+  }
+  return (w + std_shrinkf - 1) / std_shrinkf;
+}
+
+SI
+edit_typeset_rep::get_total_height (bool deco) {
+  SI h= eb->h();
+  if (!deco) {
+    SI h1= env->get_page_height (false);
+    SI h2= env->get_page_height (true);
+    int nr= get_page_count ();
+    int nx= env->page_packet;
+    int ny= ((nr + env->page_offset) + nx - 1) / nx;
+    h -= ny * (h2 - h1);
+  }
+  return (h + std_shrinkf - 1) / std_shrinkf;
 }
 
 /******************************************************************************
@@ -491,6 +571,19 @@ expand_references (tree t, hashmap<string,tree> h) {
   return r;  
 }
 
+static void
+prefix_specific (hashmap<string,tree>& H, string prefix) {
+  hashmap<string,tree> R;
+  iterator<string> it = iterate (H);
+  while (it->busy()) {
+    string key= it->next ();
+    string var= prefix * key;
+    if (H->contains (var)) R(key)= H[var];
+    else R(key)= H[key];
+  }
+  H= R;
+}
+
 tree
 edit_typeset_rep::exec (tree t, hashmap<string,tree> H, bool expand_refs) {
   hashmap<string,tree> H2;
@@ -532,6 +625,19 @@ edit_typeset_rep::exec_verbatim (tree t) {
   return exec_verbatim (t, rp * 0);
 }
 
+static tree
+search_doc_title (tree t) {
+  if (is_atomic (t)) return "";
+  else if (is_compound (t, "doc-title", 1)) return t[0];
+  else {
+    for (int i=0; i<N(t); i++) {
+      tree r= search_doc_title (t[i]);
+      if (r != "") return r;
+    }
+    return "";
+  }
+}
+
 tree
 edit_typeset_rep::exec_html (tree t, path p) {
   t= convert_OTS1_symbols_to_universal_encoding (t);
@@ -541,7 +647,11 @@ edit_typeset_rep::exec_html (tree t, path p) {
   tree patch= as_tree (eval ("(stree->tree (tmhtml-env-patch))"));
   hashmap<string,tree> P (UNINIT, patch);
   H->join (P);
+  prefix_specific (H, "tmhtml-");
   tree w (WITH);
+  tree doc_title= search_doc_title (t);
+  if (doc_title != "")
+    w << string ("html-doc-title") << doc_title;
   if (H->contains ("html-title"))
     w << string ("html-title") << H["html-title"];
   if (H->contains ("html-css"))
@@ -550,6 +660,15 @@ edit_typeset_rep::exec_html (tree t, path p) {
     w << string ("html-head-javascript") << H["html-head-javascript"];
   if (H->contains ("html-head-javascript-src"))
     w << string ("html-head-javascript-src") << H["html-head-javascript-src"];
+  if (H->contains ("html-extra-css"))
+    w << string ("html-extra-css") << H["html-extra-css"];
+  if (H->contains ("html-extra-javascript-src"))
+    w << string ("html-extra-javascript-src")
+      << H["html-extra-javascript-src"];
+  if (H->contains ("html-extra-javascript"))
+    w << string ("html-extra-javascript") << H["html-extra-javascript"];
+  if (H->contains ("html-site-version"))
+    w << string ("html-site-version") << H["html-site-version"];
   if (N(w) == 0) return exec (t, H);
   else {
     w << t;
@@ -600,6 +719,7 @@ edit_typeset_rep::exec_latex (tree t, path p) {
   tree patch= as_tree (call ("stree->tree", call ("tmtex-env-patch", t, l)));
   hashmap<string,tree> P (UNINIT, patch);
   H->join (P);
+  prefix_specific (H, "tmlatex-");
 
   if (!expand_user_macro &&
       is_document (t) && is_compound (t[0], "hide-preamble")) {
@@ -625,6 +745,42 @@ edit_typeset_rep::texmacs_exec (tree t) {
   return ::texmacs_exec (env, t);
 }
 
+tree
+edit_typeset_rep::var_texmacs_exec (tree t) {
+  typeset_exec_until (tp);
+  env->write_env (cur[tp]);
+  env->update_frame ();
+  return texmacs_exec (t);
+}
+
+/******************************************************************************
+* Wrappers for editing animations
+******************************************************************************/
+
+tree
+edit_typeset_rep::checkout_animation (tree t) {
+  path p= search_upwards (ANIM_STATIC);
+  if (is_nil (p)) p= search_upwards (ANIM_DYNAMIC);
+  if (is_nil (p)) p= search_upwards ("anim-edit");
+  if (!is_nil (p)) {
+    typeset_exec_until (p);
+    env->write_env (cur[p]);
+  }
+  return env->checkout_animation (t);
+}
+
+tree
+edit_typeset_rep::commit_animation (tree t) {
+  path p= search_upwards ("anim-edit");
+  if (is_nil (p)) p= search_upwards (ANIM_STATIC);
+  if (is_nil (p)) p= search_upwards (ANIM_DYNAMIC);
+  if (!is_nil (p)) {
+    typeset_exec_until (p);
+    env->write_env (cur[p]);
+  }
+  return env->commit_animation (t);
+}
+
 /******************************************************************************
 * Initialization
 ******************************************************************************/
@@ -662,6 +818,7 @@ void
 edit_typeset_rep::init_env (string var, tree by) {
   if (init (var) == by) return;
   init (var)= by;
+  if (var == "full-screen-mode") return;
   if (var != PAGE_SCREEN_WIDTH &&
       var != PAGE_SCREEN_HEIGHT &&
       var != ZOOM_FACTOR)
@@ -713,7 +870,7 @@ edit_typeset_rep::typeset_sub (SI& x1, SI& y1, SI& x2, SI& y2) {
 static void
 report_missing (hashmap<string,tree> missing) {
   array<string> a;
-  for (iterator<string> it= iterate (missing); it->busy(); a << it->next ());
+  for (iterator<string> it= iterate (missing); it->busy(); a << it->next ()) {}
   merge_sort (a);
   for (int i=0; i<N(a); i++)
     if (!starts (a[i], "bib-"))
@@ -732,14 +889,30 @@ report_redefined (array<tree> redefined) {
   }
 }
 
+static void
+clean_unused (hashmap<string,tree>& refs, hashmap<string,bool> used) {
+  array<string> a;
+  for (iterator<string> it= iterate (refs); it->busy(); ) {
+    string key= it->next ();
+    if (!used->contains (key)) a << key;
+  }
+  for (int i=0; i<N(a); i++)
+    refs->reset (a[i]);
+}
+
 void
 edit_typeset_rep::typeset (SI& x1, SI& y1, SI& x2, SI& y2) {
   int missing_nr= INT_MAX;
   int redefined_nr= INT_MAX;
+  x1= MAX_SI; y1= MAX_SI; x2= MIN_SI; y2= MIN_SI;
   while (true) {
-    typeset_sub (x1, y1, x2, y2);
+    SI sx1, sy1, sx2, sy2;
+    typeset_sub (sx1, sy1, sx2, sy2);
+    x1= min (x1, sx1); y1= min (y1, sy1);
+    x2= max (x2, sx2); y2= max (y2, sy2);
     if (!env->complete) break;
     env->complete= false;
+    clean_unused (env->local_ref, env->touched);
     if (N(env->missing) == 0 && N(env->redefined) == 0) break;
     if ((N(env->missing) == missing_nr && N(env->redefined) == redefined_nr) ||
         (N(env->missing) > missing_nr || N(env->redefined) > redefined_nr)) {
@@ -753,6 +926,13 @@ edit_typeset_rep::typeset (SI& x1, SI& y1, SI& x2, SI& y2) {
   }
 }
 
+void
+edit_typeset_rep::typeset_forced () {
+  //cout << "Typeset forced\n";
+  SI x1, y1, x2, y2;
+  typeset (x1, y1, x2, y2);
+}
+
 void
 edit_typeset_rep::typeset_invalidate (path p) {
   if (rp <= p) {
@@ -769,3 +949,21 @@ edit_typeset_rep::typeset_invalidate_all () {
   typeset_preamble ();
   ::notify_assign (ttt, path(), subtree (et, rp));
 }
+
+void
+edit_typeset_rep::typeset_invalidate_players (path p, bool reattach) {
+  if (rp <= p) {
+    tree t= subtree (et, p);
+    blackbox bb;
+    bool ok= t->obs->get_contents (ADDENDUM_PLAYER, bb);
+    if (ok) {
+      if (reattach) tree_addendum_delete (t, ADDENDUM_PLAYER);
+      typeset_invalidate (p);
+    }
+    if (is_compound (t)) {
+      int i, n= N(t);
+      for (i=0; i<n; i++)
+        typeset_invalidate_players (p * i, reattach);
+    }
+  }
+}
