@@ -61,19 +61,19 @@ edit_text_rep::correct_concat (path p, int done) {
       return;
     }
     else if (is_multi_paragraph (t[i]) &&
-	     is_document (subtree (et, path_up (p))))
+             is_document (subtree (et, path_up (p))))
       {
-	if ((i+1)<n) {
-	  split (p * (i+1));
-	  correct_concat (path_inc (p));
-	}
-	if (i==0) remove_node (p * 0);
-	else {
-	  split (p * i);
-	  remove_node (path_inc (p) * 0);
-	  correct_concat (p);
-	}
-	return;
+        if ((i+1)<n) {
+          split (p * (i+1));
+          correct_concat (path_inc (p));
+        }
+        if (i==0) remove_node (p * 0);
+        else {
+          split (p * i);
+          remove_node (path_inc (p) * 0);
+          correct_concat (p);
+        }
+        return;
       }
   }
 }
@@ -96,7 +96,9 @@ edit_text_rep::pure_line (path p) {
   return
     is_document (st) ||
     ((is_func (st, WITH) || is_func (st, LOCUS) ||
-      is_func (st, CANVAS) || is_func (st, ORNAMENT)) &&
+      is_func (st, CANVAS) ||
+      is_func (st, ORNAMENT) || is_func (st, ART_BOX) ||
+      is_func (st, WITH_PACKAGE)) &&
      (last_item (p) == (N(st)-1)) && pure_line (p)) ||
     (is_extension (st) && (last_item (p) >= 0) && pure_line (p));
 }
@@ -114,9 +116,11 @@ edit_text_rep::accepts_return (path p) {
     (is_func (st, XMACRO, 2) && (last_item (p) == 1)) ||
     ((is_func (st, WITH) || is_mod_active (st) ||
       is_func (st, STYLE_WITH) || is_func (st, VAR_STYLE_WITH) ||
-      is_func (st, LOCUS) ||
-      is_func (st, CANVAS) || is_func (st, ORNAMENT)) &&
+      is_func (st, LOCUS) || is_func (st, WITH_PACKAGE) ||
+      is_func (st, CANVAS)) &&
      (last_item (p) == (N(st)-1)) && pure_line (p)) ||
+    ((is_func (st, ORNAMENT) || is_func (st, ART_BOX)) &&
+     (last_item (p) == 0) && pure_line (p)) ||
     (is_extension (st) && (last_item (p) >= 0) && pure_line (p));
 }
 
@@ -170,12 +174,12 @@ edit_text_rep::prepare_for_insert () {
       insert_node (p * 0, DOCUMENT);
     else {
       if (l == 0) {
-	insert (p, tree (DOCUMENT, ""));
-	go_to (p * 0);
+        insert (p, tree (DOCUMENT, ""));
+        go_to (p * 0);
       }
       else {
-	insert (path_inc (p), tree (DOCUMENT, ""));
-	go_to (path_inc (p) * 0);
+        insert (path_inc (p), tree (DOCUMENT, ""));
+        go_to (path_inc (p) * 0);
       }
     }
     return prepare_for_insert ();
@@ -263,7 +267,7 @@ get_unit (tree t) {
   int i;
   string s= as_string (t);
   for (i=0; i<N(s); i++)
-    if ((s[i]>='a') && (s[i]<='z')) break;
+    if (is_locase (s[i])) break;
   return s (i, N(s));
 }
 
@@ -272,7 +276,7 @@ get_quantity (tree t) {
   int i;
   string s= as_string (t);
   for (i=0; i<N(s); i++)
-    if ((s[i]>='a') && (s[i]<='z')) break;
+    if (is_locase (s[i])) break;
   return s (0, i);
 }
 
@@ -361,11 +365,12 @@ edit_text_rep::make_image (
   tree t (IMAGE);
   if (link) {
     if (is_rooted (image, "default")) image= reroot (image, "file");
-    t << as_string (image, URL_STANDARD);
+    // here we convert the filename to cork-universal because it is exposed in the document, in the <image> tag
+    t << utf8_to_cork (as_string (image, URL_STANDARD));
   }
   else {
     string s;
-    tree vim= verbatim (as_string (image));
+    tree vim= verbatim (utf8_to_cork (as_string (image)));
     load_string (relative (get_name (), image), s, false);
     if (s == "") {
       set_message (concat ("File '", vim, "' not found"), "make image");
