@@ -29,7 +29,7 @@
 #  include "../Style/Memorizer/memorizer.hpp"
 #endif
 #include "new_data.hpp"
-#define TEXMACS_COPYRIGHT (string("(c) 1999-2013 by Joris van der Hoeven and others"))
+#define TEXMACS_COPYRIGHT (string("(c) 1999-2019 by Joris van der Hoeven and others"))
 
 #define THE_CURSOR 1
 #define THE_FOCUS 2
@@ -39,6 +39,7 @@
 #define THE_DECORATIONS 32
 #define THE_EXTENTS 64
 #define THE_LOCUS 128
+#define THE_MENUS 256
 
 class tm_buffer_rep;
 class tm_view_rep;
@@ -142,6 +143,7 @@ public:
   virtual void suspend () = 0;
   virtual void resume () = 0;
   virtual void keyboard_focus_on (string field) = 0;
+  virtual void update_menus () = 0;
   virtual int  get_pixel_size () = 0;
   virtual SI   get_visible_width () = 0;
   virtual SI   get_visible_height () = 0;
@@ -256,6 +258,7 @@ public:
   virtual string   multiply_length (double x, string l) = 0;
   virtual bool     is_length (string s) = 0;
   virtual double   divide_lengths (string l1, string l2) = 0;
+  virtual void     init_update () = 0;
   virtual void     drd_update () = 0;
 #ifdef EXPERIMENTAL
   virtual void     environment_update () = 0;
@@ -274,8 +277,12 @@ public:
   virtual double   get_env_double (string var_name) = 0;
   virtual double   get_init_double (string var_name) = 0;
   virtual language get_env_language () = 0;
-  virtual SI       get_page_width () = 0;
-  virtual SI       get_page_height () = 0;
+  virtual int      get_page_count () = 0;
+  virtual SI       get_page_width (bool deco) = 0;
+  virtual SI       get_pages_width (bool deco) = 0;
+  virtual SI       get_page_height (bool deco) = 0;
+  virtual SI       get_total_width (bool deco) = 0;
+  virtual SI       get_total_height (bool deco) = 0;
   virtual tree     exec_texmacs (tree t, path p) = 0;
   virtual tree     exec_texmacs (tree t) = 0;
   virtual tree     exec_verbatim (tree t, path p) = 0;
@@ -285,6 +292,9 @@ public:
   virtual tree     exec_latex (tree t, path p) = 0;
   virtual tree     exec_latex (tree t) = 0;
   virtual tree     texmacs_exec (tree t) = 0;
+  virtual tree     var_texmacs_exec (tree t) = 0;
+  virtual tree     checkout_animation (tree t) = 0;
+  virtual tree     commit_animation (tree t) = 0;
   virtual tree     get_style () = 0;
   virtual void     set_style (tree t) = 0;
   virtual void     init_style () = 0;
@@ -297,8 +307,10 @@ public:
   virtual void     set_att (string key, tree im) = 0;
   virtual void     reset_att (string key) = 0;
   virtual array<string> list_atts () = 0;
+  virtual void     typeset_forced () = 0;
   virtual void     typeset_invalidate (path p) = 0;
   virtual void     typeset_invalidate_all () = 0;
+  virtual void     typeset_invalidate_players (path p, bool reattach) = 0;
 
   /* public routines from edit_modify */
   virtual void notify_assign (path p, tree u) = 0;
@@ -476,9 +488,12 @@ public:
   virtual void selection_get (path& start, path& end) = 0;
   virtual path selection_get_start () = 0;
   virtual path selection_get_end () = 0;
+  virtual path selection_var_get_start () = 0;
+  virtual path selection_var_get_end () = 0;
   virtual path selection_get_path () = 0;
   virtual path selection_get_cursor_path () = 0;
   virtual tree selection_get_env_value (string var) = 0;
+  virtual tree selection_get (string key) = 0;
   virtual void selection_set (string key, tree t, bool persistant= false) = 0;
   virtual void selection_set (tree t) = 0;
   virtual void selection_set_start (path p= path()) = 0;
@@ -543,13 +558,14 @@ public:
   virtual void focus_on_this_editor () = 0;
   virtual void notify_page_change () = 0;
   virtual string get_metadata (string kind) = 0;
-  virtual void print (url ps_name, bool to_file, int first, int last) = 0;
+  virtual int  nr_pages () = 0;
+  virtual void print_doc (url ps_name, bool to_file, int first, int last) = 0;
   virtual void print_to_file (url ps_name,
 			      string first="1", string last="1000000") = 0;
   virtual void print_buffer (string first="1", string last="1000000") = 0;
   virtual void export_ps (url ps_name,
 			  string first="1", string last="1000000") = 0;
-  virtual array<int> print_snippet (url u, tree t) = 0;
+  virtual array<int> print_snippet (url u, tree t, bool conserve_preamble) = 0;
   virtual bool graphics_file_to_clipboard (url output) = 0;
   virtual void footer_eval (string s) = 0;
   virtual tree the_line () = 0;
@@ -597,4 +613,15 @@ editor new_editor (server_rep* sv, tm_buffer buf);
   set_current_view (temp);            \
 }
 
+// global variables for showing keypresses
+extern bool          kbd_show_keys;
+extern array<string> kbd_shown_keys;
+extern array<string> kbd_last_keys;
+extern array<time_t> kbd_last_times;
+extern int           kbd_erase_delay;
+extern int           kbd_hide_delay;
+
+bool get_show_kbd ();
+void set_show_kbd (bool flag);
+
 #endif // defined EDITOR_H
