@@ -65,6 +65,7 @@ debug_set (string s, bool on) {
   else if (s == "packrat") debug_set (DEBUG_FLAG_PACKRAT, on);
   else if (s == "flatten") debug_set (DEBUG_FLAG_FLATTEN, on);
   else if (s == "correct") debug_set (DEBUG_FLAG_CORRECT, on);
+  else if (s == "convert") debug_set (DEBUG_FLAG_CONVERT, on);
 }
 
 static bool
@@ -87,6 +88,7 @@ debug_get (string s) {
   else if (s == "packrat") return debug_get (DEBUG_FLAG_PACKRAT);
   else if (s == "flatten") return debug_get (DEBUG_FLAG_FLATTEN);
   else if (s == "correct") return debug_get (DEBUG_FLAG_CORRECT);
+  else if (s == "convert") return debug_get (DEBUG_FLAG_CONVERT);
   else return false;
 }
 
@@ -132,7 +134,8 @@ debug_message_sub (string channel, string msg) {
 void
 debug_message (string channel, string msg) {
   debug_message_sub (channel, msg);
-  if (texmacs_started) call ("notify-debug-message", object (channel));
+  if (texmacs_started && channel != "debug-widgets")
+    call ("notify-debug-message", object (channel));
 }
 
 void
@@ -140,7 +143,8 @@ debug_formatted (string channel, tree msg) {
   int n= N(debug_messages);
   if (n>0 && is_tuple (debug_messages[n-1], channel)) {
     debug_messages[n-1][2]= msg;
-    if (texmacs_started) call ("notify-debug-message", object (channel));
+    if (texmacs_started && channel != "debug-widgets")
+      call ("notify-debug-message", object (channel));
   }
 }
 
@@ -274,7 +278,7 @@ os_win32 () {
 
 bool
 os_mingw () {
-#if defined (__MINGW__) || defined (__MINGW32__)
+#ifdef OS_MINGW
   return true;
 #else
   return false;
@@ -309,6 +313,8 @@ default_look_and_feel_impl () {
   if (os_mingw () || os_win32 ()) return "windows";
   if (os_macos ()) return "macos";
   string session= get_env ("DESKTOP_SESSION");
+  if (session == "") session= get_env ("XDG_CURRENT_DESKTOP");
+  session= locase_all (session);
   if (occurs ("gnome", session))    return "gnome";
   if (occurs ("cinnamon", session)) return "gnome";
   if (occurs ("mate", session))     return "gnome";
@@ -316,7 +322,8 @@ default_look_and_feel_impl () {
   if (occurs ("kde", session))      return "kde";
   if (occurs ("xfce", session))     return "xfce";
   if (occurs ("lxde", session))     return "lxde";
-  return "emacs";
+  //return "emacs";
+  return "gnome"; // default UI (much more "standard" than emacs) 
 }
 
 const char*
