@@ -215,7 +215,7 @@ cg_renderer_rep::image (url u, SI w, SI h, SI x, SI y, int alpha) {
   // Given an image of original size (W, H),
   // we display it at position (x, y) in a rectangle of size (w, h)
 
-  // if (DEBUG_EVENTS) cout << "cg_renderer_rep::image " << as_string(u) << LF;
+  // if (DEBUG_EVENTS) debug_events << "cg_renderer_rep::image " << as_string(u) << LF;
   (void) alpha;
 
   w= w/pixel; h= h/pixel;
@@ -481,7 +481,7 @@ xpm_init (url file_name) {
   skip_spaces (s, i);
   ok= read_int (s, i, b) && ok;
   if ((!ok) || (N(t)<(c+1)) || (c<=0)) {
-    cerr << "File name= " << file_name << "\n";
+    failed_error << "File name= " << file_name << "\n";
     FAILED ("invalid xpm");
   }
 	
@@ -511,7 +511,7 @@ xpm_init (url file_name) {
       def= locase_all (s (j, i));
     }
     
-    pmcs(name)= xpm_to_color(def);
+    pmcs(name)= xpm_color(def);
   }
   CGContextRef ic = MyCreateBitmapContext(w,h);
   CGContextSetBlendMode(ic,kCGBlendModeCopy);
