@@ -10,54 +10,41 @@
 ******************************************************************************/
 
 #include "QTMWidget.hpp"
-#include "qt_widget.hpp"
-#include "qt_renderer.hpp"
 #include "qt_gui.hpp"
 #include "qt_utilities.hpp"
 #include "qt_simple_widget.hpp"
 #include "converter.hpp"
+#include "boot.hpp"
+#include "scheme.hpp"
 
 #include "config.h"
-#include "message.hpp" 
-
-#ifdef USE_CAIRO
-#include "Cairo/cairo_renderer.hpp"
-#include "Cairo/tm_cairo.hpp"
-
-#if defined (Q_WS_X11)
-#include <QX11Info>
-extern Drawable qt_x11Handle (const QPaintDevice *pd);
-extern const QX11Info *qt_x11Info (const QPaintDevice *pd);
-#undef KeyPress  // conflict between QEvent::KeyPress and X11 defintion
-#endif // Q_WS_X11
-
-#endif // USE_CAIRO
 
+#include <QDebug>
 #include <QEvent>
-#include <QSet>
 #include <QResizeEvent>
 #include <QKeyEvent>
 #include <QPaintEvent>
 #include <QMouseEvent>
 #include <QFocusEvent>
+#include <QPainter>
+#include <QApplication>
 
-#define PIXEL 256
-
-QSet<QTMWidget*> QTMWidget::all_widgets;
+#include <QBuffer>
+#include <QMimeData>
+#include <QByteArray>
+#include <QImage>
+#include <QUrl>
+#include <QFileInfo>
 
 
 hashmap<int,string> qtkeymap (0);
 hashmap<int,string> qtdeadmap (0);
 
-inline void
-scale (QPoint& point) {
-  point.rx() *= PIXEL; point.ry() *= -PIXEL;
-}
-
 inline void
 map (int code, string name) {
   qtkeymap (code) = name;
 }
+
 inline void
 deadmap (int code, string name) {
   qtdeadmap (code) = name;
@@ -65,6 +52,10 @@ deadmap (int code, string name) {
 
 void
 initkeymap () {
+  static bool fInit= false;
+  if (fInit) return;
+  fInit= true;
+  if (DEBUG_QT && DEBUG_KEYBOARD) debug_qt << "Initializing keymap\n";
   map (Qt::Key_Space     , "space");
   map (Qt::Key_Return    , "return");
   map (Qt::Key_Tab       , "tab");
@@ -126,6 +117,7 @@ initkeymap () {
   map (Qt::Key_Select    , "select" );
   map (Qt::Key_Execute   , "execute" );
   map (Qt::Key_Help      , "help" );
+  map (Qt::Key_section   , "section" );
 
   deadmap (Qt::Key_Dead_Acute     , "acute");
   deadmap (Qt::Key_Dead_Grave     , "grave");
@@ -153,7 +145,22 @@ initkeymap () {
   // map (Qt::Key_Find        , "find" );
   // map (Qt::Key_ModeSwitchFunctionKey, "modeswitch" );
 }
-
+#ifdef OS_MINGW
+enum WindowsNativeModifiers {
+    ShiftLeft            = 0x00000001,
+    ControlLeft          = 0x00000002,
+    AltLeft              = 0x00000004,
+    MetaLeft             = 0x00000008,
+    ShiftRight           = 0x00000010,
+    ControlRight         = 0x00000020,
+    AltRight             = 0x00000040,
+    MetaRight            = 0x00000080,
+    CapsLock             = 0x00000100,
+    NumLock              = 0x00000200,
+    ScrollLock           = 0x00000400,
+    ExtendedKey          = 0x01000000,
+};
+#endif
 static long int QTMWcounter = 0; // debugging hack
 
 /*! Constructor.
@@ -162,29 +169,23 @@ static long int QTMWcounter = 0; // debugging hack
   \param _tmwid the TeXmacs widget who owns this object.
  */
 QTMWidget::QTMWidget (QWidget* _parent, qt_widget _tmwid)
-: QTMScrollView (_parent), tmwid (_tmwid), backingPixmap (1,1), imwidget (NULL),
-  id (QTMWcounter++)
+: QTMScrollView (_parent), tmwid (_tmwid),  imwidget (NULL)
 {
-  setObjectName (to_qstring ("QTMWidget" * as_string (id)));// What is this for?
+  setObjectName (to_qstring ("QTMWidget" * as_string (QTMWcounter++)));// What is this for? (maybe only debugging?)
   setFocusPolicy (Qt::StrongFocus);
   setAttribute (Qt::WA_InputMethodEnabled);
-  all_widgets.insert (this);
-
-  backing_pos = origin ();
-  surface()->setMouseTracking (true);
+  surface ()->setMouseTracking (true);
+  surface ()->setAcceptDrops (true);
   
   if (DEBUG_QT)
-    debug_qt << "Creating QTMWidget " << as_string (id) << " of widget "
+    debug_qt << "Creating " << from_qstring(objectName()) << " of widget "
              << (tm_widget() ? tm_widget()->type_as_string() : "NULL") << LF;
 }
 
 QTMWidget::~QTMWidget () {
   if (DEBUG_QT)
-    debug_qt << "Destroying QTMWidget " << as_string (id) << " of widget "
+    debug_qt << "Destroying " << from_qstring(objectName()) << " of widget "
              << (tm_widget() ? tm_widget()->type_as_string() : "NULL") << LF;
-  
-    // remove ourselves from the list of QWidgets to be repainted.
-  all_widgets.remove (this);
 }
 
 qt_simple_widget_rep*
@@ -192,204 +193,6 @@ QTMWidget::tm_widget () const {
   return concrete_simple_widget (tmwid); 
 }
 
-void 
-QTMWidget::invalidate_rect (int x1, int y1, int x2, int y2) {
-#ifdef Q_WS_MAC
-  //HACK: for unknown reasons we need to enlarge the invalid rect to prevent
-  //artifacts while moving the cursor (for example at the end of a formula like
-  // $a+f$. These artifacts seems present only on 64 bit Macs. 
-  rectangle r = rectangle (x1-10, y1-10, x2+10, y2+10);
-#else
-  rectangle r = rectangle (x1, y1, x2, y2);
-#endif
-  // cout << "invalidating " << r << LF;
-  invalid_regions = invalid_regions | rectangles (r);
-}
-
-void 
-QTMWidget::invalidate_all () {
-  QSize sz = surface()->size();
- // QPoint pt = QAbstractScrollArea::viewport()->pos();
-   //cout << "invalidate all " << LF;
-  invalid_regions = rectangles();
-  invalidate_rect (0, 0, sz.width(), sz.height());
-}
-
-bool
-QTMWidget::is_invalid () {
-  return !is_nil (invalid_regions);
-}
-
-basic_renderer_rep* 
-QTMWidget::getRenderer() {
-#ifdef USE_CAIRO
-  cairo_renderer_rep *ren = the_cairo_renderer ();
-  cairo_surface_t *surf;
-#ifdef Q_WS_X11
-  //const QX11Info & info = x11Info();//qt_x11Info (this);
-  //    Display *dpy = x11Info().display();
-  //backingPixmap = QPixmap (width(),height());
-  //cout << backingPixmap.width() << LF;
-  Display *dpy = QX11Info::display();
-  Drawable drawable = backingPixmap.handle();
-  Visual *visual = (Visual*)(backingPixmap.x11Info().visual());
-  surf = tm_cairo_xlib_surface_create (dpy, drawable, visual, 
-                            backingPixmap.width (), backingPixmap.height ());
-#elif defined (Q_WS_MAC)
-  surf = tm_cairo_quartz_surface_create_for_cg_context (
-                    (CGContextRef)(this->macCGHandle()), width(), height());
-#endif
-  cairo_t *ct = tm_cairo_create (surf);
-  ren->begin (ct);
-  tm_cairo_surface_destroy (surf);
-  tm_cairo_destroy (ct);
-#else
-  qt_renderer_rep * ren = the_qt_renderer();
-  ren->begin (&backingPixmap);
-#endif
-  return ren;
-}
-
-/*
- This function is called by the qt_gui::update method to keep the backing
- store in sync and propagate the changes to the surface on screen.
- First we check that the backing store geometry is right and then we
- request to the texmacs canvas widget to repaint the regions which were
- marked invalid. Subsequently, for each succesfully repainted region, we
- propagate its contents from the backing store to the onscreen surface.
- If repaint has been interrupted we do not propagate the changes and proceed
- to mark the region invalid again.
-*/
-void
-QTMWidget::repaint_invalid_regions () {
-
-  QRegion qrgn; 
-  // qrgn is to keep track of the area on the screen which needs to be updated 
-
-  // update backing store origin wrt. TeXmacs document
-  if (backing_pos != origin()) {
-
-    int dx =  origin().x() - backing_pos.x();
-    int dy =  origin().y() - backing_pos.y();
-    backing_pos = origin();
-    
-    QPixmap newBackingPixmap (backingPixmap.size());
-    QPainter p (&newBackingPixmap);
-    //newBackingPixmap.fill (Qt::black);
-    p.drawPixmap (-dx,-dy,backingPixmap);
-    p.end();
-    backingPixmap = newBackingPixmap;
-    //cout << "SCROLL CONTENTS BY " << dx << " " << dy << LF;
-    
-    QSize sz = backingPixmap.size();
-    
-    rectangles invalid;
-    while (!is_nil (invalid_regions)) {
-      rectangle r = invalid_regions->item ;
-      //      rectangle q = rectangle (r->x1+dx,r->y1-dy,r->x2+dx,r->y2-dy);
-      rectangle q = rectangle (r->x1-dx,r->y1-dy,r->x2-dx,r->y2-dy);
-      invalid = rectangles (q, invalid);
-      //cout << r << " ---> " << q << LF;
-      invalid_regions = invalid_regions->next;
-    }
-    invalid_regions= invalid & rectangles (rectangle (0,0,
-                                                    sz.width(),sz.height()));
-    
-    if (dy<0) 
-      invalidate_rect (0,0,sz.width(),min (sz.height(),-dy));
-    else if (dy>0)
-      invalidate_rect (0,max (0,sz.height()-dy),sz.width(),sz.height());
-    
-    if (dx<0) 
-      invalidate_rect (0,0,min (-dx,sz.width()),sz.height());
-    else if (dx>0)
-      invalidate_rect (max (0,sz.width()-dx),0,sz.width(),sz.height());
-    
-    // we call update now to allow repainting of invalid regions
-    // this cannot be done directly since interpose_handler needs
-    // to be run at least once in some situations
-    // (for example when scrolling is initiated by TeXmacs itself)
-    //the_gui->update();
-    //  QAbstractScrollArea::viewport()->scroll (-dx,-dy);
-   // QAbstractScrollArea::viewport()->update();
-    qrgn += QRect (QPoint (0,0),sz);
-  }
-  
-    //cout << "   repaint QPixmap of size " << backingPixmap.width() << " x " 
-    // << backingPixmap.height() << LF;
-  // update backing store size
-  {
-    QSize _oldSize = backingPixmap.size();
-    QSize _newSize = surface()->size();
-    
-      //cout << "      surface size of " << _newSize.width() << " x " 
-      // << _newSize.height() << LF;
-    
-    
-    if (_newSize != _oldSize) {
-      // cout << "RESIZING BITMAP"<< LF;
-      QPixmap newBackingPixmap (_newSize);
-      QPainter p (&newBackingPixmap);
-      p.drawPixmap (0,0,backingPixmap);
-      //p.fillRect (0, 0, _newSize.width(), _newSize.height(), Qt::red);
-      if (_newSize.width() >= _oldSize.width()) {
-        invalidate_rect (_oldSize.width(), 0, _newSize.width(), _newSize.height());
-        p.fillRect (QRect (_oldSize.width(), 0, _newSize.width()-_oldSize.width(), _newSize.height()), Qt::gray);
-      }
-      if (_newSize.height() >= _oldSize.height()) {
-        invalidate_rect (0,_oldSize.height(), _newSize.width(), _newSize.height());
-        p.fillRect (QRect (0,_oldSize.height(), _newSize.width(), _newSize.height()-_oldSize.height()), Qt::gray);
-      }
-      p.end();
-      backingPixmap = newBackingPixmap;
-    }
-  }
-  
-  // repaint invalid rectangles
-  {
-    rectangles new_regions;
-    if (!is_nil (invalid_regions)) {
-      rectangle lub= least_upper_bound (invalid_regions);
-      if (area (lub) < 1.2 * area (invalid_regions))
-        invalid_regions= rectangles (lub);
-      
-      basic_renderer_rep* ren = getRenderer();
-      
-      SI ox = -backing_pos.x()*PIXEL;  // Warning: this is NOT from_qpoint()
-      SI oy = backing_pos.y()*PIXEL;
-      
-      rectangles rects = invalid_regions;
-      invalid_regions = rectangles();
-
-      while (!is_nil (rects)) {
-        rectangle r = copy (rects->item);
-        rectangle r0 = rects->item;
-        QRect qr = QRect (r0->x1, r0->y1, r0->x2 - r0->x1, r0->y2 - r0->y1);
-        //cout << "repainting " << r0 << "\n";
-        ren->set_origin (ox, oy); 
-        ren->encode (r->x1, r->y1);
-        ren->encode (r->x2, r->y2);
-        ren->set_clipping (r->x1, r->y2, r->x2, r->y1);
-        tm_widget()->handle_repaint (ren, r->x1, r->y2, r->x2, r->y1);
-        if (gui_interrupted ()) {
-          //cout << "interrupted repainting of  " << r0 << "\n";
-          //ren->set_pencil (green);
-          //ren->line (r->x1, r->y1, r->x2, r->y2);
-          //ren->line (r->x1, r->y2, r->x2, r->y1);
-          invalidate_rect (r0->x1, r0->y1, r0->x2, r0->y2);
-        } 
-        qrgn += qr;
-        rects = rects->next;
-      }
-      
-      ren->end();
-    } // !is_nil (invalid_regions)
-  }
-
-  // propagate immediately the changes to the screen
-  surface()->repaint (qrgn);
-}
-
 void 
 QTMWidget::scrollContentsBy (int dx, int dy) {
   QTMScrollView::scrollContentsBy (dx,dy);
@@ -437,19 +240,34 @@ QTMWidget::paintEvent (QPaintEvent* event) {
   QVector<QRect> rects = event->region().rects();
   for (int i = 0; i < rects.count(); ++i) {
     QRect qr = rects.at (i);
-    p.drawPixmap (qr, backingPixmap, qr);
+    p.drawPixmap (QRect (qr.x(), qr.y(), qr.width(), qr.height()),
+                  tm_widget()->backingPixmap,
+                  QRect (retina_factor * qr.x(),
+                         retina_factor * qr.y(),
+                         retina_factor * qr.width(),
+                         retina_factor * qr.height()));
   }
 }
 
+void
+set_shift_preference (int key_code, char shifted) {
+  set_user_preference ("shift-" * as_string (key_code), string (shifted));
+}
+
+bool
+has_shift_preference (int key_code) {
+  return has_user_preference ("shift-" * as_string (key_code));
+}
+
+string
+get_shift_preference (char key_code) {
+  return get_user_preference ("shift-" * as_string (key_code));
+}
+
 void
 QTMWidget::keyPressEvent (QKeyEvent* event) {
   if (is_nil (tmwid)) return;
-  static bool fInit = false;
-  if (!fInit) {
-    if (DEBUG_QT && DEBUG_KEYBOARD) debug_qt << "Initializing keymap\n";
-    initkeymap();
-    fInit= true;
-  }
+  initkeymap();
 
   if (DEBUG_QT && DEBUG_KEYBOARD) debug_qt << "keypressed\n";
   {
@@ -458,8 +276,13 @@ QTMWidget::keyPressEvent (QKeyEvent* event) {
 
     if (DEBUG_QT && DEBUG_KEYBOARD) {
       debug_qt << "key  : " << key << LF;
-      debug_qt << "text : " << event->text().toAscii().data() << LF;
+      debug_qt << "text : " << event->text().toLatin1().data() << LF;
       debug_qt << "count: " << event->text().count() << LF;
+#ifdef OS_MINGW
+      debug_qt << "nativeScanCode: " << event->nativeScanCode() << LF; 
+      debug_qt << "nativeVirtualKey: " << event->nativeVirtualKey() << LF;
+      debug_qt << "nativeModifiers: " << event->nativeModifiers() << LF;
+#endif
       if (mods & Qt::ShiftModifier) debug_qt << "shift\n";
       if (mods & Qt::MetaModifier) debug_qt << "meta\n";
       if (mods & Qt::ControlModifier) debug_qt << "control\n";
@@ -468,22 +291,21 @@ QTMWidget::keyPressEvent (QKeyEvent* event) {
     }
 
     string r;
-#ifdef __MINGW32__
-    // denis begin
-    if (event->text().count() == 1) {
-      QChar c= event->text()[0];
-      if (c.isPrint() && event->modifiers() != Qt::MetaModifier) {
-        // not a control character or dead key or modifier
-        char ac=c.toAscii();
-        if (ac && ac != ' ') { // a true ascii printable
-          r= ac;
-          if (DEBUG_QT && DEBUG_KEYBOARD) debug_qt << "ascii key= " <<r << "\n";	
-          the_gui->process_keypress (tm_widget(), r, texmacs_time());
-          return;
-        }
-      }
+#ifdef OS_MINGW 
+/* "Qt::Key_AltGr On Windows, when the KeyDown event for this key is sent,
+* the Ctrl+Alt modifiers are also set." (excerpt from Qt doc)
+* However the AltGr key is used to obtain many symbols 
+* which should not be regarded as C-A- shortcuts.
+* (e.g. \ or @ on a French keyboard) 
+* 
+* Hence, when "native modifiers" are (ControlLeft | AltRight) 
+* we clear Qt's Ctrl+Alt modifiers
+*/
+    if ((event->nativeModifiers() & (ControlLeft | AltRight)) == (ControlLeft | AltRight)) {
+      if (DEBUG_QT && DEBUG_KEYBOARD) debug_qt << "assuming it's an AltGr key code"<<LF;
+      mods &= ~Qt::AltModifier;
+      mods &= ~Qt::ControlModifier;
     }
-    // denis end
 #endif
     if (qtkeymap->contains (key)) {
       r = qtkeymap[key];
@@ -495,15 +317,43 @@ QTMWidget::keyPressEvent (QKeyEvent* event) {
     else {
         // We need to use text(): Alt-{5,6,7,8,9} are []|{} under MacOS, etc.
       QString nss = event->text();
+      unsigned int   kc  = event->nativeVirtualKey();
       unsigned short unic= nss.data()[0].unicode();
+      /*
+      debug_qt << "key  : " << key << LF;
+      debug_qt << "text : " << event->text().toLatin1().data() << LF;
+      debug_qt << "count: " << event->text().count() << LF;
+      if (mods & Qt::ShiftModifier) debug_qt << "shift\n";
+      if (mods & Qt::MetaModifier) debug_qt << "meta\n";
+      if (mods & Qt::ControlModifier) debug_qt << "control\n";
+      if (mods & Qt::KeypadModifier) debug_qt << "keypad\n";
+      if (mods & Qt::AltModifier) debug_qt << "alt\n";
+      cout << kc << ", " << ((mods & Qt::ShiftModifier) != 0)
+           << " -> " << unic << LF;
+      */
+      if (unic > 32 && unic < 255 &&
+          (mods & Qt::ShiftModifier) != 0 &&
+          (mods & Qt::ControlModifier) == 0 &&
+          (mods & Qt::AltModifier) == 0 &&
+          (mods & Qt::MetaModifier) == 0)
+        set_shift_preference (kc, (char) unic);
       if (unic < 32 && key < 128 && key > 0) {
-        if (((char) key) >= 'A' && ((char) key) <= 'Z') {
+        // NOTE: For some reason, the 'shift' modifier key is not applied
+        // to 'key' when 'control' is pressed as well.  We perform some
+        // dirty hacking to figure out the right shifted variant of a key
+        // by ourselves...
+        if (is_upcase ((char) key)) {
           if ((mods & Qt::ShiftModifier) == 0)
-            key= (int) (key + ((int) 'a') - ((int) 'A'));
+            key= (int) locase ((char) key);
         }
+        else if (has_shift_preference (kc) &&
+                 (mods & Qt::ShiftModifier) != 0 &&
+                 (mods & Qt::ControlModifier) != 0)
+          key= (int) (unsigned char) get_shift_preference (kc) [0];
         mods &=~ Qt::ShiftModifier;
         r= string ((char) key);
-      } else {
+      }
+      else {
         switch (unic) {
           case 96:   r= "`"; 
             // unicode to cork conversion not appropriate for this case...
@@ -522,7 +372,7 @@ QTMWidget::keyPressEvent (QKeyEvent* event) {
           case 0x33e: r= "tilde"; break;
           default:
             QByteArray buf= nss.toUtf8();
-            string rr (buf.constData(), buf.count());
+            string rr (buf.constData(), buf.size());
             string tstr= utf8_to_cork (rr);
             // HACK! The encodings defined in langs/encoding and which
             // utf8_to_cork uses (via the converters loaded in
@@ -535,13 +385,13 @@ QTMWidget::keyPressEvent (QKeyEvent* event) {
             else
               r= tstr;
             if (r == "less") r= "<";
-            else if (r == "gtr")r= ">";
+            else if (r == "gtr") r= ">";
         }
 #ifdef Q_WS_MAC
           // Alt produces many symbols in Mac keyboards: []|{} etc.
-        mods &=~ Qt::AltModifier;
+        mods &= ~Qt::AltModifier; //unset Alt
 #endif
-        mods &=~ Qt::ShiftModifier;
+        mods &= ~Qt::ShiftModifier;
       }
     }
     if (r == "") return;
@@ -635,7 +485,7 @@ QTMWidget::inputMethodEvent (QInputMethodEvent* event) {
     pal.setColor (QPalette::WindowText, Qt::white);
     imwidget->setPalette (pal);
     QFont f = imwidget->font();
-    f.setPointSize (30);
+    f.setPointSize (qt_zoom (30));
     imwidget->setFont (f);
     imwidget->setMargin (5);
   }
@@ -764,16 +614,24 @@ QTMWidget::inputMethodEvent (QInputMethodEvent* event) {
     
     r = r * as_string (pos) * ":" * from_qstring (preedit_string);
   }
+  
+#if (QT_VERSION < 0x050000)
+  // hack for fixing #47338 [CJK] input disappears immediately
+  // see http://lists.gnu.org/archive/html/texmacs-dev/2017-09/msg00000.html
   if (!is_nil (tmwid))
     the_gui->process_keypress (tm_widget(), r, texmacs_time());
+#endif 
+  
   event->accept();
 }  
 
 QVariant 
 QTMWidget::inputMethodQuery (Qt::InputMethodQuery query) const {
   switch (query) {
-    case Qt::ImMicroFocus :
-      return QVariant (QRect (cursor_pos, QSize (5,5)));
+    case Qt::ImMicroFocus : {
+      const QPoint &topleft= cursor_pos - tm_widget()->backing_pos + surface()->geometry().topLeft();
+      return QVariant (QRect (topleft, QSize (5, 5)));
+    }
     default:
       return QWidget::inputMethodQuery (query);
   }
@@ -785,10 +643,10 @@ void
 QTMWidget::mousePressEvent (QMouseEvent* event) {
   if (is_nil (tmwid)) return;
   QPoint point = event->pos() + origin();
-  scale (point);
+  coord2 pt = from_qpoint(point);
   unsigned int mstate= mouse_state (event, false);
   string s= "press-" * mouse_decode (mstate);
-  the_gui -> process_mouse (tm_widget(), s, point.x (), point.y (), 
+  the_gui -> process_mouse (tm_widget(), s, pt.x1, pt.x2,  
                             mstate, texmacs_time ());
   event->accept();
 }
@@ -797,10 +655,10 @@ void
 QTMWidget::mouseReleaseEvent (QMouseEvent* event) {
   if (is_nil (tmwid)) return;
   QPoint point = event->pos() + origin();
-  scale (point);
+  coord2 pt = from_qpoint(point);
   unsigned int mstate = mouse_state (event, true);
   string s = "release-" * mouse_decode (mstate);
-  the_gui->process_mouse (tm_widget(), s, point.x(), point.y(),
+  the_gui->process_mouse (tm_widget(), s, pt.x1, pt.x2,
                             mstate, texmacs_time());
   event->accept();
 }
@@ -809,10 +667,10 @@ void
 QTMWidget::mouseMoveEvent (QMouseEvent* event) {
   if (is_nil (tmwid)) return;
   QPoint point = event->pos() + origin();
-  scale (point);
+  coord2 pt = from_qpoint(point);
   unsigned int mstate = mouse_state (event, false);
   string s = "move";
-  the_gui->process_mouse (tm_widget(), s, point.x(), point.y(),
+  the_gui->process_mouse (tm_widget(), s, pt.x1, pt.x2, 
                           mstate, texmacs_time ());
   event->accept();
 }
@@ -860,3 +718,118 @@ QTMWidget::sizeHint () const {
   if (!is_nil (tmwid)) tm_widget()->handle_get_size_hint (w, h);
   return to_qsize (w, h);
 }
+
+void 
+QTMWidget::dragEnterEvent (QDragEnterEvent *event)
+{
+  if (is_nil (tmwid)) return;
+  const QMimeData *md = event->mimeData();
+
+  if (md->hasText() ||
+      md->hasUrls() ||
+      md->hasImage() ||
+      md->hasFormat("application/pdf") ||
+      md->hasFormat("application/postscript"))
+      event->acceptProposedAction();
+}
+
+
+// cache to transfer drop data to the editor
+// via standard mouse events, see dropEvent below
+
+int drop_payload_serial  =0;
+hashmap<int, tree> payloads;
+
+void
+QTMWidget::dropEvent (QDropEvent *event)
+{
+  if (is_nil (tmwid)) return;
+  
+  QPoint point = event->pos () + origin ();
+  coord2 pt= from_qpoint (point);
+
+  //qDebug() << event;
+  tree doc (CONCAT);
+  const QMimeData *md= event->mimeData ();
+  QByteArray buf;
+
+  if (md->hasUrls ()) {
+    QList<QUrl> l= md->urls ();
+//    qDebug() << l;
+    for (int i=0; i<l.size (); i++) {
+      string name;
+#ifdef OS_MACOS
+      name= from_qstring (fromNSUrl (l[i]));
+#else
+      name= from_qstring (l[i].toLocalFile ());
+#endif
+      string extension = suffix (name);
+      if ((extension == "eps") || (extension == "ps")   ||
+          (extension == "pdf") || (extension == "png")  ||
+          (extension == "jpg") || (extension == "jpeg")) {
+        string w, h;
+        qt_pretty_image_size (url_system (name), w, h);
+        tree im (IMAGE, name, w, h, "", "");
+        doc << im;
+      } else {
+        doc << name;
+      }
+    }
+  } else if (md->hasImage ()) {
+    QBuffer qbuf (&buf);
+    QImage image= qvariant_cast<QImage> (md->imageData());
+    QSize size= image.size ();
+    qbuf.open (QIODevice::WriteOnly);
+    image.save (&qbuf, "PNG");
+    int ww= size.width (), hh= size.height ();
+    string w, h;
+    qt_pretty_image_size (ww, hh, w, h);
+    tree t (IMAGE, tree (RAW_DATA, string (buf.constData (), buf.size()), "png"),
+            w, h, "", "");
+    doc << t;
+  } else if (md->hasFormat("application/postscript")) {
+    buf= md->data("application/postscript");
+    tree t (IMAGE, tree (RAW_DATA, string (buf.constData (), buf.size ()), "ps"),
+                   "", "", "", "");
+    doc << t;
+  } else if (md->hasFormat("application/pdf")) {
+    buf= md->data("application/pdf");
+    tree t (IMAGE, tree (RAW_DATA, string (buf.constData (), buf.size ()), "pdf"),
+                   "", "", "", "");
+    doc << t;
+  }  else if (md->hasText ()) {
+    buf= md->text ().toUtf8 ();
+    doc << string (buf.constData (), buf.size ());
+  }
+
+  if (N(doc)>0) {
+    if (N(doc) == 1)
+      doc= doc[0];
+    else {
+      tree sec (CONCAT, doc[0]);
+      for (int i=1; i<N(doc); i++)
+        sec << " " << doc[i];
+      doc= sec;
+    }
+    int ticket= drop_payload_serial++;
+    payloads (ticket)= doc;
+    the_gui->process_mouse (tm_widget(), "drop", pt.x1, pt.x2,
+                            ticket, texmacs_time ());
+    event->acceptProposedAction();
+  }
+}
+
+/*
+void
+QTMWidget::wheelEvent(QWheelEvent *event) {
+  if (QApplication::keyboardModifiers() == Qt::ControlModifier) {
+    if (event->delta() > 0) {
+      the_gui->process_keypress (tm_widget(), string("C-+"), texmacs_time());
+    } else {
+      the_gui->process_keypress (tm_widget(), string("C--"), texmacs_time());
+    }
+  } else {
+    QAbstractScrollArea::wheelEvent(event);
+  }
+}
+*/
