@@ -19,6 +19,7 @@
 #include <QMenuBar>
 #include <QLayoutItem>
 
+#include "config.h"
 #include "analyze.hpp"
 #include "scheme.hpp"
 
@@ -42,24 +43,29 @@ int menu_count = 0;  // zero if no menu is currently being displayed
 list<qt_tm_widget_rep*> waiting_widgets;
 
 static void
-replaceActions (QWidget* dest, QWidget* src) {
-    //NOTE: the parent hierarchy of the actions is not modified while installing
-    //      the menu in the GUI (see qt_menu.hpp for this memory management 
-    //      policy)
+replaceActions (QWidget* dest,  QList<QAction*>* src) {
+  //NOTE: the parent hierarchy of the actions is not modified while installing
+  //      the menu in the GUI (see qt_menu.hpp for this memory management
+  //      policy)
   if (src == NULL || dest == NULL)
     FAILED ("replaceActions expects valid objects");
   dest->setUpdatesEnabled (false);
   QList<QAction *> list = dest->actions();
-  while (!list.isEmpty()) dest->removeAction (list.takeFirst());
-  list = src->actions();
-  while (!list.isEmpty()) dest->addAction (list.takeFirst());
+  for (int i = 0; i < list.count(); i++) {
+    QAction* a = list[i];
+    dest->removeAction (a);
+  }
+  for (int i = 0; i < src->count(); i++) {
+    QAction* a = (*src)[i];
+    dest->addAction(a);
+  }
   dest->setUpdatesEnabled (true);
 }
 
 static void
-replaceButtons (QToolBar* dest, QWidget* src) {
-   if (src == NULL || dest == NULL)
-     FAILED ("replaceButtons expects valid objects");
+replaceButtons (QToolBar* dest, QList<QAction*>* src) {
+  if (src == NULL || dest == NULL)
+    FAILED ("replaceButtons expects valid objects");
   dest->setUpdatesEnabled (false);
   bool visible = dest->isVisible();
   if (visible) dest->hide(); //TRICK: to avoid flicker of the dest widget
@@ -117,6 +123,10 @@ qt_tm_widget_rep::qt_tm_widget_rep(int mask, command _quit)
   QMainWindow* mw= mainwindow ();
   mw->setStyle (qtmstyle ());
   mw->menuBar()->setStyle (qtmstyle ());
+
+#if (defined(MACOS_QT_MENU))
+  mw->menuBar()->setNativeMenuBar(false);
+#endif
   
   // there is a bug in the early implementation of toolbars in Qt 4.6
   // which has been fixed in 4.6.2 (at least)
@@ -153,6 +163,15 @@ qt_tm_widget_rep::qt_tm_widget_rep(int mask, command _quit)
   //    trying to figure this out :)
   
   bar->setMinimumWidth (2);
+#ifdef Q_WS_X11
+  int min_h= (int) floor (28 * retina_scale);
+  bar->setMinimumHeight (min_h);
+#else
+  if (retina_scale > 1.0) {
+    int min_h= (int) floor (20 * retina_scale);
+    bar->setMinimumHeight (min_h);
+  }
+#endif
   mw->setStatusBar (bar);
  
   // toolbars
@@ -211,49 +230,61 @@ qt_tm_widget_rep::qt_tm_widget_rep(int mask, command _quit)
 
 #ifdef UNIFIED_TOOLBAR
 
-  mw->setUnifiedTitleAndToolBarOnMac(true);
+  if (use_unified_toolbar) {
+    mw->setUnifiedTitleAndToolBarOnMac(true);
     
-  //WARNING: dumbToolBar is the toolbar installed on the top area of the
-  //main widget which is  then unified in the title bar. 
-  //to overcome some limitations of the unified toolbar implementation we
-  //install the real toolbars as widgets in this toolbar.
+    //WARNING: dumbToolBar is the toolbar installed on the top area of the
+    //main widget which is  then unified in the title bar. 
+    //to overcome some limitations of the unified toolbar implementation we
+    //install the real toolbars as widgets in this toolbar.
   
-  dumbToolBar = mw->addToolBar("dumb toolbar");
-  dumbToolBar->setMinimumHeight(30);
+    dumbToolBar = mw->addToolBar("dumb toolbar");
+    dumbToolBar->setMinimumHeight(30);
 
-  //these are the actions related to the various toolbars to be installed in
-  //the dumb toolbar.
+    //these are the actions related to the various toolbars to be installed in
+    //the dumb toolbar.
   
-  mainToolBarAction = dumbToolBar->addWidget(mainToolBar);
-  modeToolBarAction = NULL;
+    mainToolBarAction = dumbToolBar->addWidget(mainToolBar);
+    modeToolBarAction = NULL;
 
   
-  // A ruler
-  rulerWidget = new QWidget(cw);
-  rulerWidget->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Fixed);
-  rulerWidget->setMinimumHeight(1);
-  rulerWidget->setBackgroundRole(QPalette::Mid);
-  // FIXME: how to use 112 (active) and 146 (passive)
-  rulerWidget->setVisible(false);
-  rulerWidget->setAutoFillBackground(true);
-  // rulerWidget = new QLabel("pippo", cw);
+    // A ruler
+    rulerWidget = new QWidget(cw);
+    rulerWidget->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Fixed);
+    rulerWidget->setMinimumHeight(1);
+    rulerWidget->setBackgroundRole(QPalette::Mid);
+    // FIXME: how to use 112 (active) and 146 (passive)
+    rulerWidget->setVisible(false);
+    rulerWidget->setAutoFillBackground(true);
+    // rulerWidget = new QLabel("pippo", cw);
   
     // A second ruler (this one always visible) to separate from the canvas.
-  QWidget* r2 = new QWidget(mw);
-  r2->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Fixed);
-  r2->setMinimumHeight(1);
-  r2->setBackgroundRole(QPalette::Mid);
-  r2->setVisible(true);
-  r2->setAutoFillBackground(true);
+    QWidget* r2 = new QWidget(mw);
+    r2->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Fixed);
+    r2->setMinimumHeight(1);
+    r2->setBackgroundRole(QPalette::Mid);
+    r2->setVisible(true);
+    r2->setAutoFillBackground(true);
 
-  bl->insertWidget(0, modeToolBar);
-  bl->insertWidget(1, rulerWidget);
-  bl->insertWidget(2, focusToolBar);
-  bl->insertWidget(3, userToolBar);
-  bl->insertWidget(5, r2);
+    bl->insertWidget(0, modeToolBar);
+    bl->insertWidget(1, rulerWidget);
+    bl->insertWidget(2, focusToolBar);
+    bl->insertWidget(3, userToolBar);
+    bl->insertWidget(5, r2);
 
     //mw->setContentsMargins (-2, -2, -2, -2);  // Why this?
-  bar->setContentsMargins (0, 1, 0, 1);
+    bar->setContentsMargins (0, 1, 0, 1);
+  }
+  else {
+    mw->addToolBar (mainToolBar);
+    mw->addToolBarBreak ();
+    mw->addToolBar (modeToolBar);
+    mw->addToolBarBreak ();
+    mw->addToolBar (focusToolBar);
+    mw->addToolBarBreak ();
+    mw->addToolBar (userToolBar);
+    mw->addToolBarBreak ();
+  }
 
 #else
   mw->addToolBar (mainToolBar);
@@ -315,6 +346,19 @@ qt_tm_widget_rep::~qt_tm_widget_rep () {
 
 void
 qt_tm_widget_rep::tweak_iconbar_size (QSize& sz) {
+#ifdef Q_WS_X11
+  if (sz.height () >= 24) {
+    sz.setWidth (sz.width () + 2);
+    sz.setHeight (sz.height () + 8);
+  }
+  else if (sz.height () >= 20) {
+    sz.setWidth (sz.width () + 1);
+    sz.setHeight (sz.height () + 4);
+  }
+  else if (sz.height () >= 16) {
+    sz.setHeight (sz.height () + 4);
+  }
+#else
   if (sz.height () >= 24) {
     sz.setWidth (sz.width () + 2);
     sz.setHeight (sz.height () + 6);
@@ -325,6 +369,8 @@ qt_tm_widget_rep::tweak_iconbar_size (QSize& sz) {
   else if (sz.height () >= 16) {
     sz.setHeight (sz.height () + 2);
   }
+#endif
+  //sz.setHeight ((int) floor (sz.height () * retina_scale + 0.5));
 }
 
 
@@ -385,8 +431,9 @@ qt_tm_widget_rep::update_visibility () {
 #ifdef UNIFIED_TOOLBAR
 
   // do modifications only if needed to reduce flicker
-  if ( XOR(old_mainVisibility,  new_mainVisibility) ||
-      XOR(old_modeVisibility,  new_modeVisibility) )
+  if (use_unified_toolbar &&
+      (XOR(old_mainVisibility,  new_mainVisibility) ||
+       XOR(old_modeVisibility,  new_modeVisibility) ))
   {
     // ensure that the topmost visible toolbar is always unified on Mac
     // (actually only for main and mode toolbars, unifying focus is not
@@ -436,7 +483,7 @@ qt_tm_widget_rep::update_visibility () {
   {
     QFont f = leftLabel->font();
     int fs = as_int (get_preference ("gui:mini-fontsize", QTM_MINI_FONTSIZE));
-    f.setPointSize (fs > 0 ? fs : QTM_MINI_FONTSIZE);
+    f.setPointSize (qt_zoom (fs > 0 ? fs : QTM_MINI_FONTSIZE));
     leftLabel->setFont(f);
     rightLabel->setFont(f);
   }
@@ -708,48 +755,26 @@ qt_tm_widget_rep::query (slot s, int type_id) {
 
 void
 qt_tm_widget_rep::install_main_menu () {
+  if (main_menu_widget == waiting_main_menu_widget) return;
   main_menu_widget = waiting_main_menu_widget;
-  QMenu* src = main_menu_widget->get_qmenu();
+  QList<QAction*>* src = main_menu_widget->get_qactionlist();
   if (!src) return;
-
-    // REMARK: We do not want the menubar shared across windows as suggested
-    // in http://doc.qt.nokia.com/4.7/qmainwindow.html#menuBar
-    // e.g. :
-    //
-    //     QMenuBar *dest = new QMenuBar(0);
-    //     mainwindow()->setMenuBar(dest);
-    //
-    // as the default behavior on MacOS. The main reason is that in TeXmacs
-    // different windows can have different main menus so that it is indeed
-    // appropriate to change the main menu as the window focus changes.
-    // So we kindly ask to each window to give us its own menu and we install
-    // there our actions.
-    // So we do:
-  
   QMenuBar* dest = mainwindow()->menuBar();
-  
-    // and everything is fine.
-  
-    // Also please note that we have to do the replacement and not simply
-    // install the menu returned by get_qmenu() since the main menu there
-    // could contain some default items appropriate for the given OS (like the
-    // service menu on MacOS) which are not present in our menu widget.
-    // UPDATE: But replaceActions() *does remove* all actions (!?)
-    // It looks more like the reason is that we have to "flatten out" the first
-    // level of the QMenu returned... ?
-  
-  replaceActions (dest, src);
-  QList<QAction*> list = dest->actions();
-  for (int i = 0; i < list.count(); i++) {
-    QAction* a = list[i];
+  dest->clear();
+  for (int i = 0; i < src->count(); i++) {
+    QAction* a = (*src)[i];
     if (a->menu()) {
+      //TRICK: Mac native QMenuBar accepts only menus which are already populated
+      // this will cause a problem for us, since menus are lazy and populated only after triggering
+      // this is the reason we add a dummy action before inserting the menu
+      a->menu()->addAction("native menubar trick");
+      dest->addAction(a->menu()->menuAction());
       QObject::connect (a->menu(),         SIGNAL (aboutToShow()),
                         the_gui->gui_helper, SLOT (aboutToShowMainMenu()));
       QObject::connect (a->menu(),         SIGNAL (aboutToHide()),
                         the_gui->gui_helper, SLOT (aboutToHideMainMenu()));
     }
   }
-
 }
 
 void
@@ -803,9 +828,9 @@ qt_tm_widget_rep::write (slot s, blackbox index, widget w) {
       check_type_void (index, s);
     {
       main_icons_widget = concrete (w);
-      QMenu* m = main_icons_widget->get_qmenu();
-      if (m) {
-        replaceButtons (mainToolBar, m);
+      QList<QAction*>* list = main_icons_widget->get_qactionlist();
+      if (list) {
+        replaceButtons (mainToolBar, list);
         update_visibility();
       }
     }
@@ -815,9 +840,9 @@ qt_tm_widget_rep::write (slot s, blackbox index, widget w) {
       check_type_void (index, s);
     {
       mode_icons_widget = concrete (w);
-      QMenu* m = mode_icons_widget->get_qmenu();
-      if (m) {
-        replaceButtons (modeToolBar, m);
+      QList<QAction*>* list = mode_icons_widget->get_qactionlist();
+      if (list) {
+        replaceButtons (modeToolBar, list);
         update_visibility();
       }
     }
@@ -827,9 +852,9 @@ qt_tm_widget_rep::write (slot s, blackbox index, widget w) {
       check_type_void (index, s);
     {
       focus_icons_widget = concrete (w);
-      QMenu* m = focus_icons_widget->get_qmenu();
-      if (m) {
-        replaceButtons (focusToolBar, m);
+      QList<QAction*>* list = focus_icons_widget->get_qactionlist();
+      if (list) {
+        replaceButtons (focusToolBar, list);
         update_visibility();
       }
     }
@@ -839,9 +864,9 @@ qt_tm_widget_rep::write (slot s, blackbox index, widget w) {
       check_type_void (index, s);
     {   
       user_icons_widget = concrete (w);
-      QMenu* m = user_icons_widget->get_qmenu();
-      if (m) {
-        replaceButtons (userToolBar, m);
+      QList<QAction*>* list = user_icons_widget->get_qactionlist();
+      if (list) {
+        replaceButtons (userToolBar, list);
         update_visibility();
       }
     }
@@ -894,30 +919,40 @@ qt_tm_widget_rep::set_full_screen(bool flag) {
   QWidget *win = mainwindow()->window();  
   if (win) {
     if (flag ) {
+      QPalette pal;
+      pal.setColor(QPalette::Mid, Qt::black);
+      mainwindow()->setPalette(pal);
 #ifdef UNIFIED_TOOLBAR
-      //HACK: we disable unified toolbar since otherwise
-      //  the application will crash when we return to normal mode
-      // (bug in Qt? present at least with 4.7.1)
-      mainwindow()->setUnifiedTitleAndToolBarOnMac(false);
-      mainwindow()->centralWidget()->layout()->setContentsMargins(0,0,0,0);
+      if (use_unified_toolbar) {
+        //HACK: we disable unified toolbar since otherwise
+        //  the application will crash when we return to normal mode
+        // (bug in Qt? present at least with 4.7.1)
+        mainwindow()->setUnifiedTitleAndToolBarOnMac(false);
+        mainwindow()->centralWidget()->layout()->setContentsMargins(0,0,0,0);
+      }
 #endif
 //      mainwindow()->window()->setContentsMargins(0,0,0,0);
       //win->showFullScreen();
-       win->setWindowState(win->windowState() ^ Qt::WindowFullScreen);
-    } else {
+       win->setWindowState(win->windowState() | Qt::WindowFullScreen);
+    }
+    else {
+      QPalette pal;
+      mainwindow()->setPalette(pal);
       bool cache = visibility[0];
       visibility[0] = false;
       update_visibility();
 //      win->showNormal();
-      win->setWindowState(win->windowState() ^ Qt::WindowFullScreen);
+      win->setWindowState(win->windowState() & ~Qt::WindowFullScreen);
 
       visibility[0] = cache;
       update_visibility();
 #ifdef UNIFIED_TOOLBAR
-      mainwindow()->centralWidget()->layout()->setContentsMargins (0,1,0,0);
-      //HACK: we reenable unified toolbar (see above HACK) 
-      //  the application will crash when we return to normal mode
-      mainwindow()->setUnifiedTitleAndToolBarOnMac(true);
+      if (use_unified_toolbar) {
+        mainwindow()->centralWidget()->layout()->setContentsMargins (0,1,0,0);
+        //HACK: we reenable unified toolbar (see above HACK) 
+        //  the application will crash when we return to normal mode
+        mainwindow()->setUnifiedTitleAndToolBarOnMac(true);
+      }
 #endif
     }
   }
