@@ -148,6 +148,7 @@ input_widget_rep::continuous () {
   return
     starts (type, "search") ||
     starts (type, "replace-") ||
+    starts (type, "spell") ||
     starts (serial, "form-");
 }
 
@@ -155,16 +156,20 @@ void
 input_widget_rep::handle_get_size (get_size_event ev) {
   SI ww = decode_length (width, this, style);
   SI dww= (SI) ((2*dw) / SHRINK);
-  if (ends (width, "w") && is_double (width (0, N(width) - 1))) {
+  string w_unit;
+  double w_len;
+  parse_length (width, w_len, w_unit);
+
+  if (w_unit == "w") {
     font fn= get_default_styled_font (style);
     if (ev->mode == -1) ev->w= 0;
     else if (ev->mode == 0);
     else if (ev->mode == 1) ev->w= ww;
     ev->w= max (ev->w, (4*fn->wquad) / SHRINK + dww);
   }
-  else if (ends (width, "em") && is_double (width (0, N(width) - 2)))
+  else if (w_unit == "em")
     ev->w= ww + dww;
-  else if (ends (width, "px") && is_double (width (0, N(width) - 2)))
+  else if (w_unit == "px")
     ev->w= ww + dww;
   else if (ev->mode == 1)
     ev->w= ww;
@@ -295,7 +300,9 @@ input_widget_rep::handle_keypress (keypress_event ev) {
        key == "pagedown" ||
        key == "tab" ||
        key == "S-tab" ||
-       key == "escape"));
+       key == "escape" ||
+       (starts (type, "spell") && string ("1") <= key && key <= string ("9")) ||
+       (starts (type, "spell") && key == "+")));
   else if (key == "return") commit ();
   else if ((key == "escape") || (key == "C-c") ||
 	   (key == "C-g")) cancel ();
