@@ -18,8 +18,9 @@
 #include "drd_std.hpp"
 #include "language.hpp"
 #include <unistd.h>
-#ifdef __MINGW32__
+#ifdef OS_MINGW
 #include <time.h>
+#include <direct.h>
 #endif
 
 tree texmacs_settings = tuple ();
@@ -85,7 +86,7 @@ plugin_list () {
 
 static void
 init_main_paths () {
-#ifdef __MINGW32__
+#ifdef OS_MINGW
   if (is_none (get_env_path ("TEXMACS_HOME_PATH", get_env ("APPDATA") * "/TeXmacs"))) {
 #else
   if (is_none (get_env_path ("TEXMACS_HOME_PATH", "~/.TeXmacs"))) {
@@ -108,6 +109,8 @@ static string main_tmp_dir= "$TEXMACS_HOME_PATH/system/tmp";
 
 static void
 make_dir (url which) {
+  if (is_none(which))
+    return ;
   if (!is_directory (which)) {
     make_dir (head (which));
     mkdir (which);
@@ -116,7 +119,7 @@ make_dir (url which) {
 
 static url
 url_temp_dir_sub () {
-#ifdef __MINGW32__
+#ifdef OS_MINGW
   static url tmp_dir=
     url_system (main_tmp_dir) * url_system (as_string (time (NULL)));
 #else
@@ -147,7 +150,7 @@ static void
 clean_temp_dirs () {
   bool err= false;
   array<string> a= read_directory (main_tmp_dir, err);
-#ifndef __MINGW32__
+#ifndef OS_MINGW
   for (int i=0; i<N(a); i++)
     if (is_int (a[i]))
       if (!process_running (as_int (a[i])))
@@ -163,7 +166,7 @@ clean_temp_dirs () {
         url cur = url (main_tmp_dir) * url (a[i]);
         array<string> f= read_directory (cur, err);
         for (int j=0; j<N(f); j++) remove (cur * url (f[j]));
-        _rmdir (as_charp (as_string (cur)));
+        rmdir (as_charp (as_string (cur)));
       }
     }
 #endif
@@ -209,6 +212,7 @@ init_user_dirs () {
   make_dir ("$TEXMACS_HOME_PATH/system/cache");
   make_dir ("$TEXMACS_HOME_PATH/system/database");
   make_dir ("$TEXMACS_HOME_PATH/system/database/bib");
+  make_dir ("$TEXMACS_HOME_PATH/system/make");
   make_dir ("$TEXMACS_HOME_PATH/system/tmp");
   make_dir ("$TEXMACS_HOME_PATH/texts");
   make_dir ("$TEXMACS_HOME_PATH/users");
@@ -253,13 +257,13 @@ init_guile () {
     if (guile_version == "") {
       var_eval_system ("guile-config info top_srcdir");
       for (i=N(guile_version); i>0; i--)
-	if (guile_version[i-1] == '-') break;
+        if (guile_version[i-1] == '-') break;
       guile_version= guile_version (i, N (guile_version));
       for (i=0; i<N(guile_version); i++)
-	if ((guile_version[i] == '/') || (guile_version[i] == '\\')) {
-	  guile_version= guile_version (0, i);
-	  break;
-	}
+        if ((guile_version[i] == '/') || (guile_version[i] == '\\')) {
+          guile_version= guile_version (0, i);
+          break;
+        }
     }
     url guile_dir= url_system (guile_data) * url ("guile", guile_version);
     guile_path= guile_path | guile_dir;
@@ -290,20 +294,20 @@ init_env_vars () {
   // Get TeXmacs style and package paths
   url style_root=
     get_env_path ("TEXMACS_STYLE_ROOT",
-		  "$TEXMACS_HOME_PATH/styles:$TEXMACS_PATH/styles" |
-		  plugin_path ("styles"));
+                  "$TEXMACS_HOME_PATH/styles:$TEXMACS_PATH/styles" |
+                  plugin_path ("styles"));
   url package_root=
     get_env_path ("TEXMACS_PACKAGE_ROOT",
-		  "$TEXMACS_HOME_PATH/packages:$TEXMACS_PATH/packages" |
-		  plugin_path ("packages"));
+                  "$TEXMACS_HOME_PATH/packages:$TEXMACS_PATH/packages" |
+                  plugin_path ("packages"));
   url all_root= style_root | package_root;
   url style_path=
     get_env_path ("TEXMACS_STYLE_PATH",
                   search_sub_dirs (all_root));
   url text_root=
     get_env_path ("TEXMACS_TEXT_ROOT",
-		  "$TEXMACS_HOME_PATH/texts:$TEXMACS_PATH/texts" |
-		  plugin_path ("texts"));
+                  "$TEXMACS_HOME_PATH/texts:$TEXMACS_PATH/texts" |
+                  plugin_path ("texts"));
   url text_path=
     get_env_path ("TEXMACS_TEXT_PATH",
                   search_sub_dirs (text_root));
@@ -311,29 +315,29 @@ init_env_vars () {
   // Get other data paths
   (void) get_env_path ("TEXMACS_FILE_PATH",text_path | style_path);
   (void) set_env_path ("TEXMACS_DOC_PATH",
-		       get_env_path ("TEXMACS_DOC_PATH") |
-		       "$TEXMACS_HOME_PATH/doc:$TEXMACS_PATH/doc" |
-		       plugin_path ("doc"));
+                       get_env_path ("TEXMACS_DOC_PATH") |
+                       "$TEXMACS_HOME_PATH/doc:$TEXMACS_PATH/doc" |
+                       plugin_path ("doc"));
   (void) set_env_path ("TEXMACS_SECURE_PATH",
-		       get_env_path ("TEXMACS_SECURE_PATH") |
-		       "$TEXMACS_PATH:$TEXMACS_HOME_PATH");
+                       get_env_path ("TEXMACS_SECURE_PATH") |
+                       "$TEXMACS_PATH:$TEXMACS_HOME_PATH");
   (void) get_env_path ("TEXMACS_PATTERN_PATH",
-		       "$TEXMACS_HOME_PATH/misc/patterns" |
-		       url ("$TEXMACS_PATH/misc/patterns") |
-		       plugin_path ("misc/patterns"));
+                       "$TEXMACS_HOME_PATH/misc/patterns" |
+                       url ("$TEXMACS_PATH/misc/patterns") |
+                       plugin_path ("misc/patterns"));
   (void) get_env_path ("TEXMACS_PIXMAP_PATH",
-		       "$TEXMACS_HOME_PATH/misc/pixmaps" |
-		       url ("$TEXMACS_PATH/misc/pixmaps/modern/32x32/settings") |
-		       url ("$TEXMACS_PATH/misc/pixmaps/modern/32x32/table") |
-		       url ("$TEXMACS_PATH/misc/pixmaps/modern/24x24/main") |
-		       url ("$TEXMACS_PATH/misc/pixmaps/modern/20x20/mode") |
-		       url ("$TEXMACS_PATH/misc/pixmaps/modern/16x16/focus") |
-		       url ("$TEXMACS_PATH/misc/pixmaps/traditional/--x17") |
-		       plugin_path ("misc/pixmaps"));
+                       "$TEXMACS_HOME_PATH/misc/pixmaps" |
+                       url ("$TEXMACS_PATH/misc/pixmaps/modern/32x32/settings") |
+                       url ("$TEXMACS_PATH/misc/pixmaps/modern/32x32/table") |
+                       url ("$TEXMACS_PATH/misc/pixmaps/modern/24x24/main") |
+                       url ("$TEXMACS_PATH/misc/pixmaps/modern/20x20/mode") |
+                       url ("$TEXMACS_PATH/misc/pixmaps/modern/16x16/focus") |
+                       url ("$TEXMACS_PATH/misc/pixmaps/traditional/--x17") |
+                       plugin_path ("misc/pixmaps"));
   (void) get_env_path ("TEXMACS_DIC_PATH",
-		       "$TEXMACS_HOME_PATH/langs/natural/dic" |
-		       url ("$TEXMACS_PATH/langs/natural/dic") |
-		       plugin_path ("langs/natural/dic"));
+                       "$TEXMACS_HOME_PATH/langs/natural/dic" |
+                       url ("$TEXMACS_PATH/langs/natural/dic") |
+                       plugin_path ("langs/natural/dic"));
 #ifdef OS_WIN32
   set_env ("TEXMACS_SOURCE_PATH", "");
 #else
@@ -348,7 +352,7 @@ init_env_vars () {
 static void
 init_misc () {
   // Test whether 'which' works
-#if defined(__MINGW__) || defined(__MINGW32__) || defined (OS_WIN32)
+#ifdef OS_MINGW
   use_which = false;
 #else
   use_which = (var_eval_system ("which texmacs 2> /dev/null") != "");
@@ -368,7 +372,7 @@ init_misc () {
 static void
 setup_inkscape_extension () {
 debug_boot << "attempt install of inkscape extension \n ";
-#if defined(__MINGW__) || defined(__MINGW32__) || defined (OS_WIN32)
+#ifdef OS_MINGW
   url ink_ext = url ("$APPDATA/inkscape/extensions");
 #else
   url ink_ext = "~/.config/inkscape/extensions/";
