@@ -16,6 +16,7 @@
 #include "merge_sort.hpp"
 #include "iterator.hpp"
 #include "boot.hpp"
+#include "drd_std.hpp"
 
 int geometry_w= 800, geometry_h= 600;
 int geometry_x= 0  , geometry_y= 0;
@@ -24,6 +25,11 @@ widget texmacs_window_widget (widget wid, tree geom);
 widget make_menu_widget (object menu);
 void refresh_size (widget wid, bool exact);
 
+static int last_window_handle= 0;
+static hashmap<int,widget> window_table (NULL);
+static hashmap<tree,path> window_by_name;
+static time_t refresh_time= 0;
+
 /******************************************************************************
 * User preference management concerning the geometry of windows
 ******************************************************************************/
@@ -70,6 +76,9 @@ static bool
 resize_accept (int old_w, int old_h, int w, int h) {
   if (old_w == w && old_h == h) return false;
 #ifdef QTTEXMACS
+  if (old_w == w && old_h == h + 41) return false;
+  if (old_w == w && old_h == h + 24) return false;
+  if (old_w == w && old_h == h + 22) return false;
   if (old_w == w && old_h == h + 16) return false;
   if (old_w == w && old_h == h - 40) return false;
 #endif
@@ -117,6 +126,7 @@ get_preferred_size (string name, SI& ww, SI& hh) {
     int h= as_int (get_user_preference ("height " * name));
     ww= w * PIXEL;
     hh= h * PIXEL;
+    //cout << "Size " << name << ": " << w << ", " << h << "\n";
   }
 }
 
@@ -133,7 +143,7 @@ tm_window_rep::tm_window_rep (widget wid2, tree geom):
   menu_current (object ()), menu_cache (widget ()),
   text_ptr (NULL)
 {
-  zoomf= get_server () -> get_default_zoom_factor ();
+  zoomf= retina_zoom * get_server () -> get_default_zoom_factor ();
 }
 
 tm_window_rep::tm_window_rep (tree doc, command quit):
@@ -143,7 +153,8 @@ tm_window_rep::tm_window_rep (tree doc, command quit):
   menu_current (object ()), menu_cache (widget ()),
   text_ptr (NULL)
 {
-  zoomf= get_server () -> get_default_zoom_factor ();
+  (void) doc;
+  zoomf= retina_zoom * get_server () -> get_default_zoom_factor ();
 }
 
 tm_window_rep::~tm_window_rep () {
@@ -167,6 +178,10 @@ texmacs_window_widget (widget wid, tree geom) {
     w= as_int (geom[0]);
     h= as_int (geom[1]);
   }
+  if (w == 800 && h == 600) {
+    w *= retina_zoom;
+    h *= retina_zoom;
+  }
   gui_root_extents (W, H); W /= PIXEL; H /= PIXEL;
   if (x < 0) x= W + x + 1 - w;
   if (y < 0) y= H + y + 1 - h;
@@ -190,8 +205,12 @@ texmacs_window_widget (widget wid, tree geom) {
 
 class close_embedded_command_rep: public command_rep {
   tm_view vw;
+  url name;
+  int win_id;
 public:
-  close_embedded_command_rep (tm_view vw2): vw (vw2) {}
+  close_embedded_command_rep (tm_view vw2, url n2, int win2):
+    vw (vw2), name (n2), win_id (win2) {
+      window_by_name(name->t)= path (win_id, window_by_name[name->t]); }
   void apply ();
   tm_ostream& print (tm_ostream& out) {
     return out << "Close_Embedded widget command"; }
@@ -212,6 +231,7 @@ close_embedded_command_rep::apply () {
   tm_window win= vw->win;
   ASSERT (N (buffer_to_views (vw->buf->buf->name)) == 1,
           "invalid cloned embedded TeXmacs widget");
+  window_by_name(name->t)= remove<int> (window_by_name[name->t], win_id);
   remove_buffer (vw->buf->buf->name);
   //cout << "Deleted buffer\n";
   tm_delete (win);
@@ -219,8 +239,26 @@ close_embedded_command_rep::apply () {
 }
 
 command
-close_embedded_command (tm_view vw) {
-  return tm_new<close_embedded_command_rep> (vw);
+close_embedded_command (tm_view vw, url name, int win) {
+  return tm_new<close_embedded_command_rep> (vw, name, win);
+}
+
+static path
+filter_existing (path wins) {
+  if (is_nil (wins)) return wins;
+  if (window_table->contains (wins->item))
+    return path (wins->item, filter_existing (wins->next));
+  return filter_existing (wins->next);
+}
+
+path
+window_search (url name) {
+  return filter_existing (window_by_name[name->t]);
+}
+
+bool
+is_embedded_buffer (url name) {
+  return !is_nil (window_search (name));
 }
 
 /******************************************************************************
@@ -250,6 +288,9 @@ enrich_embedded_document (tree body, tree style) {
     for (int i=0; i+2<N(orig); i+=2)
       if (is_atomic (orig[i]))
         initial (orig[i]->label)= orig[i+1];
+  initial (DPI)= "720";
+  initial (ZOOM_FACTOR)= (retina_zoom==1? "1.2": "1.8");
+  initial ("no-zoom")= "true";
   tree doc (DOCUMENT);
   doc << compound ("TeXmacs", TEXMACS_VERSION);
   doc << style; //compound ("style", style);
@@ -273,7 +314,8 @@ texmacs_input_widget (tree doc, tree style, url wname) {
   set_scrollable (win->wid, vw->ed);
   vw->ed->cvw= win->wid.rep;
   vw->ed->mvw= curvw;
-  return wrapped_widget (win->wid, close_embedded_command (vw));
+  command close_cmd= close_embedded_command (vw, name, last_window_handle);
+  return wrapped_widget (win->wid, close_cmd);
 }
 
 /******************************************************************************
@@ -317,15 +359,17 @@ tm_window_rep::refresh () {
 * Menus
 ******************************************************************************/
 
-#ifdef QTTEXMACS
-bool menu_caching= false;
-#else
 bool menu_caching= true;
-#endif
 
 bool
 tm_window_rep::get_menu_widget (int which, string menu, widget& w) {
+  drd_info old_drd= the_drd;
+  if (!is_none (window_to_view (id))) {
+    tm_view vw= concrete_view (window_to_view (id));
+    if (vw != NULL) the_drd= vw->ed->drd;
+  }
   object xmenu= call ("menu-expand", eval ("'" * menu));
+  the_drd= old_drd;
   //cout << "xmenu= " << xmenu << "\n";
   if (menu_cache->contains (xmenu)) {
     //if (menu_current[which] == xmenu) cout << "Same " << menu << "\n";
@@ -339,7 +383,9 @@ tm_window_rep::get_menu_widget (int which, string menu, widget& w) {
   //cout << "Compute " << menu << "\n";
   object umenu= eval ("'" * menu);
   w= make_menu_widget (umenu);
-  if (menu_caching) menu_cache (xmenu)= w;
+  if (menu_caching)
+    if (as_bool (call ("cache-menu?", xmenu)))
+      menu_cache (xmenu)= w;
   return true;
 }
 
@@ -436,13 +482,13 @@ tm_window_rep::get_bottom_tools_flag (int which) {
 
 void
 tm_window_rep::set_window_zoom_factor (double zoom) {
-  zoomf= zoom;
-  ::set_zoom_factor (wid, zoom);
+  zoomf= retina_zoom * zoom;
+  ::set_zoom_factor (wid, zoomf);
 }
 
 double
 tm_window_rep::get_window_zoom_factor () {
-  return zoomf;
+  return zoomf / retina_zoom;
 }
 
 void
@@ -548,12 +594,10 @@ tm_window_rep::interactive_return () {
 * Other top level windows
 ******************************************************************************/
 
-static hashmap<int,widget> window_table (NULL);
-static time_t refresh_time= 0;
-
 int
 window_handle () {
   static int window_next= 1;
+  last_window_handle= window_next;
   return window_next++;
 }
 
