@@ -49,6 +49,7 @@ struct text_box_rep: public box_rep {
   text_box_rep (path ip, int pos, string s, font fn, pencil pen, xkerning xk);
   operator tree () { return str; }
   box adjust_kerning (int mode, double factor);
+  box expand_glyphs (int mode, double factor);
 
   void      display (renderer ren);
   double    left_slope ();
@@ -64,6 +65,7 @@ struct text_box_rep: public box_rep {
   SI        sup_lo_lim  (int level);
   SI        sup_lo_base (int level);
   SI        sup_hi_lim  (int level);
+  SI        wide_correction (int mode);
 
   path      find_box_path (SI x, SI y, SI delta, bool force, bool& found);
   path      find_lip ();
@@ -118,19 +120,30 @@ text_box_rep::adjust_kerning (int mode, double factor) {
     nxk->right= xk->right;
   }
   if ((mode & PROTRUSION_MASK) != 0) {
-    nxk->left = -fn->get_left_protrusion  (str, mode);
-    nxk->right= -fn->get_right_protrusion (str, mode);
+    if ((mode & START_OF_LINE) != 0)
+      nxk->left = -fn->get_left_protrusion  (str, mode);
+    if ((mode & END_OF_LINE) != 0)
+      nxk->right= -fn->get_right_protrusion (str, mode);
   }
   if ((mode & START_OF_LINE) != 0) nxk->left  -= pad;
   if ((mode & END_OF_LINE  ) != 0) nxk->right -= pad;
   return tm_new<text_box_rep> (ip, pos, str, fn, pen, nxk);
 }
 
+box
+text_box_rep::expand_glyphs (int mode, double factor) {
+  if (N(str) == 0) return this;
+  font nfn= fn->magnify (1.0 + factor, 1.0);
+  return tm_new<text_box_rep> (ip, pos, str, nfn, pen, xk);
+}
+
 void
 text_box_rep::display (renderer ren) {
-  ren->set_pencil (pen);
-  if (is_nil (xk)) fn->draw (ren, str, 0, 0);
-  else fn->draw (ren, str, xk->left, 0, xk->padding);
+  if (N(str) > 0) {
+    ren->set_pencil (pen);
+    if (is_nil (xk)) fn->draw (ren, str, 0, 0);
+    else fn->draw (ren, str, xk->left, 0, xk->padding);
+  }
 }
 
 double text_box_rep::left_slope () {
@@ -142,13 +155,13 @@ SI text_box_rep::left_correction () {
 SI text_box_rep::right_correction () {
   return fn->get_right_correction (str); }
 SI text_box_rep::lsub_correction () {
-  return -left_correction (); }
+  return fn->get_lsub_correction (str); }
 SI text_box_rep::lsup_correction () {
-  return right_correction (); }
+  return fn->get_lsup_correction (str); }
 SI text_box_rep::rsub_correction () {
-  return 0; }
+  return fn->get_rsub_correction (str); }
 SI text_box_rep::rsup_correction () {
-  return right_correction (); }
+  return fn->get_rsup_correction (str); }
 
 SI
 text_box_rep::sub_lo_base (int level) {
@@ -180,6 +193,12 @@ text_box_rep::sup_hi_lim  (int level) {
   return fn->ysup_hi_lim;
 }
 
+SI
+text_box_rep::wide_correction (int mode) {
+  if (mode == 0) return tm_string_length (str) <= 1;
+  else return fn->get_wide_correction (str, mode);
+}
+
 /******************************************************************************
 * New routines concerning the cursor
 ******************************************************************************/
@@ -373,7 +392,7 @@ get_delimiter (string s, font fn, SI height) {
   int ns= N(s);
   ASSERT (ns >= 2 && s[0] == '<' && s[ns-1] == '>',
 	  "invalid rubber character");
-  if (s[ns-2] >= '0' && s[ns-2] <= '9') {
+  if (is_digit (s[ns-2])) {
     int pos;
     int plus= get_number (s, pos);
     if (pos > 0) {
@@ -393,7 +412,8 @@ get_delimiter (string s, font fn, SI height) {
   SI best_h= 0;
   int n= 0;
   SI last= 0;
-  while (true) {
+  int credit= 20;
+  while (credit > 0) {
     metric ex;
     string test= radical * as_string (n) * ">";
     fn->get_extents (test, ex);
@@ -428,8 +448,9 @@ get_delimiter (string s, font fn, SI height) {
       last= h;
     }
     else return best;
+    credit--;
   }
-  return s;
+  return best;
 }
 
 static string
@@ -451,11 +472,14 @@ get_wide (string s, font fn, SI width) {
   SI  d= w2- w1;
   int n= (width-w1) / (d+1);
 
+  int credit= 20;
   while (true) {
     string test= radical * as_string (n+1) * ">";
     fn->get_extents (test, ey);
-    if (ey->x2- ey->x1 > width) return radical * as_string (n) * ">";
+    if (ey->x2- ey->x1 > width || credit <= 0)
+      return radical * as_string (n) * ">";
     n++;
+    credit--;
   }
 }
 
@@ -486,12 +510,26 @@ delimiter_box (path ip, string s, font fn, pencil pen, SI bot, SI top) {
   box b= text_box (ip, 0, r, fn, pen);
   SI x= -b->x1;
   SI y= (top + bot - b->y1 - b->y2) >> 1;
+  if (s == "<large-sqrt>") {
+    SI sep= fn->sep, Y= y + ((b->y1 + b->y2) >> 1);
+    if (Y > fn->yfrac) {
+      SI d= min (min (Y - fn->yfrac, y + b->y2 - top), 2*sep);
+      if (d > 0) y -= d;
+    }
+    else if (Y < fn->yfrac) {
+      SI d= min (min (fn->yfrac - Y, bot - (y + b->y1)), 2*sep);
+      if (d > 0) y += d;
+    }
+  }
   //cout << s << ", " << bot/PIXEL << " -- " << top/PIXEL
   //     << " -> " << r << "; " << x/PIXEL << ", " << y/PIXEL << "\n";
   //cout << "  extents: " << b->x1/PIXEL << ", " << b->y1/PIXEL
   //     << "; " << b->x2/PIXEL << ", " << b->y2/PIXEL << "\n";
-  box mvb= move_box (ip, b, x, y, false, true);
-  return macro_box (ip, mvb, fn);
+  SI var_bot= max (bot, b->y1 + y), var_top= min (top, b->y2 + y);
+  box mvb= move_delimiter_box (ip, b, x, y, var_bot, var_top);
+  if (ends (r, "-0>")) return mvb;
+  SI dy= ((mvb->y1 + mvb->y2)>>1) - fn->yfrac;
+  return macro_delimiter_box (ip, mvb, fn, dy);
 }
 
 box
@@ -512,8 +550,10 @@ delimiter_box (path ip, string s, font fn, pencil pen,
   //     << " -> " << r << "; " << x/PIXEL << ", " << y/PIXEL << "\n";
   //cout << "  extents: " << b->x1/PIXEL << ", " << b->y1/PIXEL
   //     << "; " << b->x2/PIXEL << ", " << b->y2/PIXEL << "\n";
-  box mvb= move_box (ip, b, x, y, false, true);
-  return macro_box (ip, mvb, fn);
+  box mvb= move_delimiter_box (ip, b, x, y, real_bot, real_top);
+  if (ends (r, "-0>")) return mvb;
+  SI dy= ((mvb->y1 + mvb->y2)>>1) - fn->yfrac;
+  return macro_delimiter_box (ip, mvb, fn, dy);
 }
 
 box
