@@ -44,6 +44,8 @@ class concater_rep {
   // textual markup
   void typeset_substring (string s, path ip, int pos);
   void typeset_math_substring (string s, path ip, int pos, int op_type);
+  void typeset_math_macro (string s, tree_label m, path ip,
+                           int p1, int p2, int ot);
   void typeset_colored_substring (string s, path ip, int pos, string col);
   void typeset_text_string (tree t, path ip, int start, int end);
   void typeset_math_string (tree t, path ip, int start, int end);
@@ -53,6 +55,7 @@ class concater_rep {
   void typeset_surround (tree t, path ip);
   void typeset_concat (tree t, path ip);
   void typeset_rigid (tree t, path ip);
+  void typeset_hgroup (tree t, path ip);
   void typeset_hspace (tree t, path ip);
   void typeset_space (tree t, path ip);
   void typeset_move (tree t, path ip);
@@ -60,15 +63,17 @@ class concater_rep {
   void typeset_resize (tree t, path ip);
   void typeset_clipped (tree t, path ip);
   void typeset_float (tree t, path ip);
-  void typeset_repeat (tree t, path ip);
+  void typeset_repeat (tree t, path ip, bool under);
   void typeset_formatting (tree t, path ip, string var);
   void typeset_decorated_box (tree t, path ip);
   void typeset_line_note (tree t, path ip);
   void typeset_page_note (tree t, path ip);
+  void typeset_if_page_break (tree t, path ip);
 
   // mathematical markup
   void typeset_around (tree t, path ip, bool colored);
   void typeset_large (tree t, path ip, int type, int op_type, string prefix);
+  void typeset_wide_middle (tree t, path ip);
   //void typeset_left (tree t, path ip);
   //void typeset_middle (tree t, path ip);
   //void typeset_right (tree t, path ip);
@@ -79,11 +84,14 @@ class concater_rep {
   void typeset_below (tree t, path ip);
   void typeset_above (tree t, path ip);
   void typeset_script (tree t, path ip, bool right);
+  void typeset_wide_frac (tree t, path ip);
   void typeset_frac (tree t, path ip);
+  void typeset_wide_sqrt (tree t, path ip);
   void typeset_sqrt (tree t, path ip);
   void typeset_wide (tree t, path ip, bool above);
   void typeset_neg (tree t, path ip);
   void typeset_tree (tree t, path ip);
+  void typeset_wide_table (tree t, path ip);
   void typeset_table (tree t, path ip);
   void print_semantic (box b, tree sem);
   void typeset_syntax (tree t, path ip);
@@ -100,6 +108,7 @@ class concater_rep {
 
   // active macro mechanisms
   void typeset_assign (tree t, path ip);
+  void typeset_provide (tree t, path ip);
   void typeset_with (tree t, path ip);
   void typeset_compound (tree t, path ip);
   void typeset_auto (tree t, path ip, tree macro);
@@ -107,6 +116,7 @@ class concater_rep {
   void typeset_drd_props (tree t, path ip);
   void typeset_eval (tree t, path ip);
   void typeset_value (tree t, path ip);
+  void typeset_or_value (tree t, path ip);
   void typeset_argument (tree t, path ip);
   void typeset_eval_args (tree t, path ip);
   void typeset_mark (tree t, path ip);
@@ -126,11 +136,13 @@ class concater_rep {
   void typeset_toc_notify (tree t, path ip);
   void typeset_specific (tree t, path ip);
   void typeset_flag (tree t, path ip);
+  void typeset_hyphenate_as (tree t, path ip);
 
   // animations
   void typeset_anim_compose (tree t, path ip);
   void typeset_anim_repeat (tree t, path ip);
   void typeset_anim_constant (tree t, path ip);
+  void typeset_anim_accelerate (tree t, path ip);
   void typeset_anim_translate (tree t, path ip);
   void typeset_anim_progressive (tree t, path ip);
   void typeset_video (tree t, path ip);
@@ -145,6 +157,7 @@ class concater_rep {
   void typeset_gr_effect (tree t, path ip);
   void typeset_text_at (tree t, path ip);
   void typeset_math_at (tree t, path ip);
+  void typeset_document_at (tree t, path ip);
   void typeset_point (tree t, path ip);
   array<box> typeset_line_arrows (path ip);
   void typeset_line (tree t, path ip, bool close);
@@ -152,6 +165,7 @@ class concater_rep {
   void typeset_spline (tree t, path ip, bool close);
   void typeset_var_spline (tree t, path ip);
   void typeset_cspline (tree t, path ip);
+  void typeset_bezier (tree t, path ip);
   void typeset_fill (tree t, path ip);
   void typeset_image (tree t, path ip);
   void typeset_graphics_3d (tree t, path ip);
@@ -159,11 +173,14 @@ class concater_rep {
   // markup for graphical user interface
   void typeset_canvas (tree t, path ip);
   void typeset_ornament (tree t, path ip);
+  void typeset_art_box (tree t, path ip);
 
   // postprocessing of brackets and scripts
   int  prec (int i);
   int  succ (int i);
   void pre_glue ();
+  box  glue_left_markers (box b, int ref, int arg);
+  box  glue_right_markers (box b, int ref, int arg, bool flag);
   void glue (box b, int ref, int arg);
   void glue (box b, int ref, int arg1, int arg2);
   void clean_and_correct ();
@@ -183,6 +200,8 @@ public:
   friend class document_rep;
 
   friend box              typeset_as_concat (edit_env env, tree t, path ip);
+  friend box              typeset_as_concat (edit_env env, tree t, path ip,
+                                             array<line_item>& a);
   friend box              typeset_as_box (edit_env env, tree t, path ip);
   friend box              typeset_as_atomic (edit_env env, tree t, path ip);
   friend array<line_item> typeset_concat (edit_env env, tree t, path ip);
