@@ -9,6 +9,7 @@
 * in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
+#include "page_type.hpp"
 #include "Format/page_item.hpp"
 #include "Format/stack_border.hpp"
 #include "pager.hpp"
@@ -16,12 +17,12 @@ box format_stack (path ip, array<box> bx, array<space> ht, SI height,
 		  bool may_stretch);
 #include "Boxes/construct.hpp"
 array<page_item> sub (array<page_item> l, path p, path q);
-SI stretch_space (space spc, double stretch);
 page_item access (array<page_item> l, path p);
+space as_space (tree t);
 skeleton break_pages (array<page_item> l, space ph, int qual,
 		      space fn_sep, space fnote_sep, space float_sep,
                       font fn, int first_page);
-box page_box (path ip, box b, tree page, int page_nr,
+box page_box (path ip, box b, tree page, int page_nr, brush bgc,
               SI width, SI height, SI left, SI top,
 	      SI bot, box header, box footer, SI head_sep, SI foot_sep);
 
@@ -49,9 +50,14 @@ pager_rep::pages_format (array<page_item> l, SI ht, SI tcor, SI bcor) {
       spc << item->spc;
     }
   }
-  ASSERT (N(bs) != 0, "zero lines in insertion");
-  box b= format_stack (ip, bs, spc, ht, true);
-  return vcorrect_box (b->ip, b, tcor, bcor);
+  if (N(bs) == 0) {
+    box b= empty_box (decorate_middle (ip), 0, -ht, 0, 0);
+    return vcorrect_box (b->ip, b, tcor, bcor);
+  }
+  else {
+    box b= format_stack (ip, bs, spc, ht, true);
+    return vcorrect_box (b->ip, b, tcor, bcor);
+  }
 }
 
 box
@@ -74,7 +80,8 @@ pager_rep::pages_format (insertion ins) {
   else {
     array<page_item> sub_l= sub (l, ins->begin, ins->end);
     SI ht= stretch_space (ins->ht, ins->stretch);
-    return pages_format (sub_l, ht, ins->top_cor, ins->bot_cor);
+    SI xh= stretch_space (ins->xh, ins->stretch);
+    return pages_format (sub_l, ht, ins->top_cor, ins->bot_cor + xh);
   }
 }
 
@@ -140,6 +147,11 @@ pager_rep::pages_format (pagelet pg) {
 	  // cout << "sep " << stretch_space (float_sep, ins->stretch) << LF;
 	  y -= stretch_space (float_sep, ins->stretch);
 	}
+	if (is_tuple (ins->type, "if-page-break")) {
+          // cout << "sep "
+          // << stretch_space (as_space (ins->type[2]), ins->stretch) << LF;
+          y -= stretch_space (as_space (ins->type[2]), ins->stretch);
+	}
       }
     }
     if (fnote_y != MAX_SI) {
@@ -162,11 +174,22 @@ pager_rep::pages_make_page (pagelet pg) {
   env->write (PAGE_NR, as_string (nr));
   env->write (PAGE_THE_PAGE, style[PAGE_THE_PAGE]);
   tree page_t= env->exec (compound (PAGE_THE_PAGE));
-  box header= make_header (N (pg->ins) == 0);
-  box footer= make_footer (N (pg->ins) == 0);
-  return page_box (ip, lb, page_t, nr,
-		   width, height, left, top, top+ text_height,
-		   header, footer, head_sep, foot_sep);
+  bool empty= N (pg->ins) == 0;
+  box header= make_header (empty);
+  box footer= make_footer (empty);
+  brush bgc = make_background (empty);
+  adjust_margins (empty);
+  box page= page_box (ip, lb, page_t, nr, bgc, width, height,
+                      left, top + dtop, top + dtop + text_height,
+                      header, footer, head_sep, foot_sep);
+  if (env->get_string (PAGE_CROP_MARKS) == "") return page;
+  bool ls= env->page_landscape;
+  string sz= env->get_string (PAGE_CROP_MARKS);
+  SI w= env->as_length (page_get_feature (sz, PAGE_WIDTH, ls));
+  SI h= env->as_length (page_get_feature (sz, PAGE_HEIGHT, ls));
+  SI lw= env->as_length ("0.2ln");
+  SI ll= env->as_length ("1cm");
+  return crop_marks_box (ip, page, w, h, lw, ll);
 }
 
 void
@@ -193,15 +216,17 @@ pager_rep::papyrus_make () {
 
   box sb= pages_format (sk[0]);
   box b = move_box (ip, sb, 0, 0);
+  brush bgc = make_background (false);
+  adjust_margins (false);
   SI ph= b->h();
-  if (env->get_string (PAGE_MEDIUM) == "beamer")
-    ph= max (ph, env->page_user_height);
   SI left  = (odd+even) >> 1;
-  SI height= top + bot + ph;
+  SI height= top + dtop + bot + dbot + ph;
+  if (env->get_string (PAGE_MEDIUM) == "beamer")
+    height= max (height, env->page_user_height + top + bot);
   array<box> bs   (1); bs   [0]= b;
   array<SI>  bs_x (1); bs_x [0]= left;
-  array<SI>  bs_y (1); bs_y [0]= -top;
-  box pb= page_box (ip, "?", 0, width, height,
+  array<SI>  bs_y (1); bs_y [0]= -top - dtop;
+  box pb= page_box (ip, "?", 0, bgc, width, height,
 		    bs, bs_x, bs_y, 0, 0, 0);
   pages << pb;
 }
